// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Main.cs" company="">
//   
// </copyright>
// <summary>
//   The form 1.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.Main
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Net;
    using System.Reflection;
    using System.Threading;
    using System.Windows.Forms;
    using System.Xml;

    using entity.MapForms;

    using Globals;

    using HaloMap;
    using HaloMap.Map;

    using Microsoft.Win32;

    /// <summary>
    /// The form 1.
    /// </summary>
    /// <remarks></remarks>
    partial class Form1 : Form
    {
        // Used for Entity updates
        #region Constants and Fields

        /// <summary>
        /// The update ftp name.
        /// </summary>
        private static string updateFTPName = "h2misc@acemods.org";

        /// <summary>
        /// The update ftp pass.
        /// </summary>
        private static string updateFTPPass = "y,Nk+,G/ur5i";

        /// <summary>
        /// The update ftp server.
        /// </summary>
        private static string updateFTPServer = "ftp://ftp.acemods.org/Troy Mac1ures Folder (Reaper approved)/";

        /// <summary>
        /// The update nfo file.
        /// </summary>
        private static string updateNFOFile = "update.xml";

        /// <summary>
        /// The update upd file.
        /// </summary>
        private static string updateUpdFile = "UpdateEnt.exe";

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        /// <remarks></remarks>
        public Form1()
        {
            InitializeComponent();

            this.Text = Assembly.GetExecutingAssembly().GetName().Name + "  " +
                        Assembly.GetExecutingAssembly().GetName().Version.Major + "." +
                        Assembly.GetExecutingAssembly().GetName().Version.Minor + "." +
                        Assembly.GetExecutingAssembly().GetName().Version.Build;

            // load preferences
            if (Prefs.Load())
            {
                this.AddRecentFilesToMenu();
            }
            else
            {
                MessageBox.Show(
                        "Error: \"" + Prefs.FilePath + "\" not found.\n" +
                        "Please setup default map directories first.");
                Settings settings = new Settings();
                settings.ShowDialog();
            }

            #region check Update

            bool checkUpdate = false;
            TimeSpan tSpan;
            tSpan = DateTime.Now.Subtract(Prefs.lastCheck);
            switch (Prefs.checkUpdate)
            {
                case Prefs.updateFrequency.Always:
                    checkUpdate = true;
                    break;
                case Prefs.updateFrequency.Daily:
                    if (tSpan.Days > 0)
                    {
                        checkUpdate = true;
                    }

                    break;
                case Prefs.updateFrequency.Weekly:
                    if (tSpan.Days > 6)
                    {
                        checkUpdate = true;
                    }

                    break;
                case Prefs.updateFrequency.Monthly:
                    if (tSpan.Days > 30)
                    {
                        checkUpdate = true;
                    }

                    break;
            }

            if (checkUpdate)
            {
                Thread thr = new Thread(IsThereAnUpdate);
                thr.Start(false);
            }

            #endregion

            #region Load any dropped maps

            string[] temp = Environment.GetCommandLineArgs();
            if (temp.Length > 1)
            {
                TryLoadMapForm(temp[1]);
            }

            #endregion
        }

        #endregion

        #region Delegates

        /// <summary>
        /// The delegate which we will call from the thread to update the form
        /// </summary>
        /// <param name="BytesRead">The bytes read.</param>
        /// <param name="TotalBytes">The total bytes.</param>
        /// <remarks></remarks>
        private delegate void UpdateProgessCallback(long BytesRead, long TotalBytes);

        #endregion

        #region Properties

        /// <summary>
        /// Count of open maps
        /// </summary>
        /// <remarks></remarks>
        public int MapCount { get; private set; }

        #endregion

        #region Public Methods

        /// <summary>
        /// The reload skins.
        /// </summary>
        /// <remarks></remarks>
        public void ReloadSkins()
        {
            foreach (MapForm mapForm in this.MdiChildren)
            {
                mapForm.LoadSkin();
            }
        }

        /// <summary>
        /// Attempts to load a mapand bring up a map form for
        /// it. Function will fail if the map is corrupt and
        /// display an error message. Provide a null string to
        /// show openfileDialog. Returns a refernce to the created mapform
        /// if needed
        /// </summary>
        /// <param name="mapFileName">The map File Name.</param>
        /// <returns></returns>
        /// <remarks></remarks>
        public MapForm TryLoadMapForm(string mapFileName)
        {
            // Only allow for 10 map forms to be open at any one time, for
            // memory reasons
            if (MapCount >= 10)
            {
                MessageBox.Show("Too many maps open, close a map.");
                return null;
            }

            // show a dialog if null filename
            if (mapFileName == null)
            {
                // if the user cancels out the dialog, no map to open
                if (openmapdialog.ShowDialog() == DialogResult.Cancel)
                {
                    return null;
                }

                // set the file name to user chosen file
                mapFileName = openmapdialog.FileName;
            }

            // Check map isn't already loaded into a map form
            foreach (MapForm mapForm in this.MdiChildren)
            {
                if (mapForm.map.filePath == mapFileName)
                {
                    MessageBox.Show("This map is already open in this editor!..");
                    return mapForm;
                }
            }

            // Attempt to load the map
            Map newMap = Map.LoadFromFile(mapFileName);

            // map failed to load for some reason
            if (newMap == null)
            {
                MessageBox.Show("Map failed to load...");
                return null;
            }

            // create a map form for the map
            MapForm newMapForm = new MapForm(newMap);

            // set it to a child of this main form
            newMapForm.MdiParent = this;

            // keep track of open maps
            MapCount++;
            if (MapCount > 0)
            {
                closemapmenu.Enabled = true;
            }

            // store map in recent list
            UpdateRecentFiles(mapFileName);

            // give the form a close event
            newMapForm.FormClosed += newMapForm_FormClosed;

            // show form
            newMapForm.Show();

            return newMapForm;
        }

        #endregion

        #region Methods

        /// <summary>
        /// The form 1_ drag drop.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void Form1_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
                foreach (string s in files)
                {
                    TryLoadMapForm(s);
                }
            }
        }

        /// <summary>
        /// The form 1_ drag enter.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void Form1_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Move;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        /// <summary>
        /// The is there an update.
        /// </summary>
        /// <param name="showInf">The show inf.</param>
        /// <exception cref="WebException">
        ///   </exception>
        /// <remarks></remarks>
        private void IsThereAnUpdate(object showInf)
        {
            bool showInfo = (bool)showInf;
            WebClient wc = new WebClient();

            // Set our FTP Username & Password
            wc.Credentials = new NetworkCredential(updateFTPName, updateFTPPass);

            // Download our Information File
            // [0] = Version Number
            // [1] = Build Number
            // [2] = Release Date
            // [3] = Update Filename
            // [4] = Change Log Filename
            // [5] = Informational URL
            // [6] = Updater Version
            int count = 0;
            string netVerInfo = null;
            while (netVerInfo == null)
            {
                try
                {
                    netVerInfo = wc.DownloadString(updateFTPServer + updateNFOFile);
                }
                catch (WebException e)
                {
                    // Give 5 chances to connect
                    count++;
                    if (count < 5)
                    {
                        Thread.Sleep(300);
                    }
                    else
                    {
                        if (showInfo) throw e;
                    }
                }
            }

            string updtVersion = string.Empty;
            string updtDate = string.Empty;
            string updtName = string.Empty;

            XmlDocument xd = new XmlDocument();
            xd.LoadXml(netVerInfo);
            if (xd.FirstChild.Name == "entityupdater")
            {
                XmlNode xn = xd.FirstChild.FirstChild;
                while (xn != null)
                {
                    switch (xn.Name)
                    {
                        case "updateinfo":
                            XmlNode relInfoNode = xn.FirstChild;
                            while (relInfoNode != null)
                            {
                                switch (relInfoNode.Name)
                                {
                                    case "version":
                                        updtVersion = relInfoNode.InnerText.Replace(" ", string.Empty);
                                        break;
                                    case "date":
                                        updtDate = relInfoNode.InnerText;
                                        break;
                                    case "filename":
                                        updtName = relInfoNode.InnerText;
                                        break;
                                }

                                relInfoNode = relInfoNode.NextSibling;
                            }

                            break;
                    }

                    xn = xn.NextSibling;
                }
            }

            try
            {
                if (!File.Exists(Global.StartupPath + "\\" + updateUpdFile))
                {
                    wc.DownloadFile(updateFTPServer + updateUpdFile, Global.StartupPath + "\\" + updateUpdFile);
                }
                else
                {
                    // Get current info of our updater
                    FileVersionInfo updtFile = FileVersionInfo.GetVersionInfo(Global.StartupPath + "\\" + updateUpdFile);

                    int VerMaj = updtFile.FileMajorPart;
                    int VerMin = updtFile.FileMinorPart;
                    string[] s = updtVersion.Split('.');
                    if (int.Parse(s[0]) > VerMaj || (int.Parse(s[0]) == VerMaj && int.Parse(s[1]) > VerMin))
                    {
                        File.Delete(Global.StartupPath + "\\" + updateUpdFile);
                        wc.DownloadFile(updateFTPServer + updateUpdFile, Global.StartupPath + "\\" + updateUpdFile);
                    }
                }
            }
            catch
            {
            }

            wc.Dispose();

            Prefs.lastCheck = DateTime.Now;
            Prefs.Save();

            // Run update checker
            string args = "/pid:" + Process.GetCurrentProcess().Id;

            // Attach any open maps to be reopened
            foreach (MapForm mapForm in this.MdiChildren)
            {
                args += " /om:" + mapForm.map.filePath;
            }

            Process.Start(Global.StartupPath + "\\" + updateUpdFile, args);
        }

        /// <summary>
        /// Adds the recent files to menu.
        /// </summary>
        /// <remarks></remarks>
        private void AddRecentFilesToMenu()
        {
            int i = 0;
            foreach (Prefs.RecentFile recentMap in Prefs.RecentOpenedMaps)
            {
                MenuItem mItem = new MenuItem();
                mItem.Index = 8 + i;
                mItem.Shortcut = (Shortcut)((int)Shortcut.Alt1 + i);
                mItem.Text = recentMap.Path;
                mItem.Tag = recentMap;
                mItem.Click += this.lastFile_Click;
                recentMap.MenuItem = mItem;
                this.menuFile.MenuItems.Add(mItem);

                // Seperator
                menuItem1.Visible = true;

                i++;
            }
        }

        /// <summary>
        /// The update recent files.
        /// </summary>
        /// <param name="newFile">The new file.</param>
        /// <remarks></remarks>
        private void UpdateRecentFiles(string newFile)
        {
            Prefs.RecentFile rf = null;

            bool alreadyExists = false;
            foreach (Prefs.RecentFile recentMap in Prefs.RecentOpenedMaps)
            {
                if (newFile == recentMap.Path)
                {
                    rf = recentMap;
                    alreadyExists = true;
                    break;
                }
            }

            if (alreadyExists)
            {
                menuFile.MenuItems.Remove(rf.MenuItem);
                Prefs.RecentOpenedMaps.Remove(rf);
            }
            else
            {
                rf = new Prefs.RecentFile();
                rf.Path = newFile;
            }

            Prefs.RecentOpenedMaps.Insert(0, rf);

            if (Prefs.RecentOpenedMaps.Count > Prefs.MaxRecentFiles)
            {
                menuFile.MenuItems.Remove(Prefs.RecentOpenedMaps[Prefs.RecentOpenedMaps.Count - 1].MenuItem);
                Prefs.RecentOpenedMaps.RemoveAt(Prefs.RecentOpenedMaps.Count - 1);
            }

            this.AddRecentFilesToMenu();
        }

        /// <summary>
        /// The close entity_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void closeEntity_Click(object sender, EventArgs e)
        {
            ActiveForm.Close();
        }

        /// <summary>
        /// The closemapmenu_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void closemapmenu_Click(object sender, EventArgs e)
        {
            // Wrong Sender. MenuItem, not MapWindow....hmmm...

            // the menu item is the sender because that is the control that called this method ~Prey
            MapForm mapForm = (MapForm)this.ActiveMdiChild;
            mapForm.Close();
            mapForm.Dispose();
        }

        /// <summary>
        /// The help about_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void helpAbout_Click(object sender, EventArgs e)
        {
            AboutBox ab = new AboutBox();
            ab.ShowDialog();
        }

        /// <summary>
        /// The help ve controls_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void helpVEControls_Click(object sender, EventArgs e)
        {
            MessageBox.Show(
                "CONTROLS FOR THE VISUAL EDITOR\n" + "==============================\n" + " W - Camera Forwards\n" +
                " S - Camera Backwards\n" + " A - Camera Left\n" + " D - Camera Right\n" + " Z - Camera Down\n" +
                " X - Camera Up\n" + "\n" + " Right Mouse Button - Camera Look-Around\n" +
                " Left  Mouse Button - Select/Deselect Object/Spawn\n" + "\n" + "WHEN OBJECT/SPAWN SELECTED\n" +
                "==========================\n" + " Up/Down    Arrows = Move Object Along X-Axis\n" +
                " Left/Right Arrows = Move Object Along Y-Axis\n" + " Page Up/Page Down = Move Object Along Z-Axis\n" +
                "\n" + " SHIFT + Left Mouse Button - Rotate Object Yaw (Around Z-Axis)\n" +
                " ALT   + Left Mouse Button - Rotate Object Pitch (Around Y-Axis)\n" +
                " CTRL  + Left Mouse Button - Rotate Object Roll (Around X-Axis)\n", 
                "BSP Visual Editor Controls");
        }

        /// <summary>
        /// The last file_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void lastFile_Click(object sender, EventArgs e)
        {
            TryLoadMapForm(((MenuItem)sender).Text);
        }

        /// <summary>
        /// The mainmenueditmenu item 7_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void mainmenueditmenuItem7_Click(object sender, EventArgs e)
        {
            MainmenuEdit mme = new MainmenuEdit();
            mme.ShowDialog();
        }

        /// <summary>
        /// The makedefault map editor menu item_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void makedefaultMapEditorMenuItem_Click(object sender, EventArgs e)
        {
            if (
                MessageBox.Show(
                    "Are you sure you would like to set Entity as your default editor?", 
                    "Set As Default Editor", 
                    MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                RegistryKey regKey;
                regKey = Registry.ClassesRoot.CreateSubKey("Entity\\");
                regKey.SetValue(string.Empty, "Halo Map File");
                regKey = Registry.ClassesRoot.CreateSubKey("Entity\\shell\\open\\command");
                regKey.SetValue(string.Empty, Application.ExecutablePath + " %1");
                regKey = Registry.ClassesRoot.CreateSubKey(".map");
                regKey.SetValue(string.Empty, "Entity");
                regKey.Close();
                MessageBox.Show("Entity is now your default map editor.", "Set Default Editor");
            }
        }

        /// <summary>
        /// The new map form_ form closed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void newMapForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            MapForm mapForm = (MapForm)sender;

            // make sure map stream is closed
            mapForm.map.CloseMap();

            // update map count
            MapCount--;
            if (MapCount <= 0)
            {
                closemapmenu.Enabled = false;
            }

            Prefs.Save();
        }

        /// <summary>
        /// The newmapmenu_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void newmapmenu_Click(object sender, EventArgs e)
        {
            MapWizard newWizard = new MapWizard();
            newWizard.Owner = this;
            newWizard.Text = "Create a new map...";
            newWizard.Show();
        }

        /// <summary>
        /// The openmapmenu_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void openmapmenu_Click(object sender, EventArgs e)
        {
            // pass in null so the function shows an open file dialog
            // for the user to choose a map
            TryLoadMapForm(null);
        }

        /// <summary>
        /// The tile cascademenu item 5_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void tileCascademenuItem5_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.Cascade);
        }

        /// <summary>
        /// The tile horizontalmenu item 3_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void tileHorizontalmenuItem3_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.TileHorizontal);
        }

        /// <summary>
        /// The tile verticalmenu item 4_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void tileVerticalmenuItem4_Click(object sender, EventArgs e)
        {
            this.LayoutMdi(MdiLayout.TileVertical);
        }

        /// <summary>
        /// The tools check for updates_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void toolsCheckForUpdates_Click(object sender, EventArgs e)
        {
            try
            {
                IsThereAnUpdate(true);
            }
            catch (WebException webEx)
            {
                switch (webEx.Status)
                {
                    case WebExceptionStatus.NameResolutionFailure:
                        Global.ShowErrorMsg(webEx.Message, webEx);
                        break;
                    default:
                        Global.ShowErrorMsg("Internet Error", webEx);
                        break;
                }
            }
        }

        /// <summary>
        /// The tools format plugins_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void toolsFormatPlugins_Click(object sender, EventArgs e)
        {
            folderBrowserDialog1.SelectedPath = Global.StartupPath;
            if (folderBrowserDialog1.ShowDialog() == DialogResult.Cancel)
            {
                return;
            }

            MiscFunctions.FormatIFPsInDirectory(folderBrowserDialog1.SelectedPath);
            MessageBox.Show("Converted IFP and IFP2 Files");
        }

        /// <summary>
        /// The tools settings_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void toolsSettings_Click(object sender, EventArgs e)
        {
            Settings settings = new Settings();
            settings.ShowDialog();
        }

        /// <summary>
        /// The tools skin options_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void toolsSkinOptions_Click(object sender, EventArgs e)
        {
            SkinOptions skinopt = new SkinOptions();
            skinopt.mainform = this;
            skinopt.Show();
        }

        /// <summary>
        /// The tools video settings_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void toolsVideoSettings_Click(object sender, EventArgs e)
        {
            VideoSettings vs = new VideoSettings();
            vs.ShowDialog();
        }

        #endregion

        /// <summary>
        /// Handles the FormClosing event of the Form1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        /// <remarks></remarks>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Prefs.Save();
        }
    }
}