﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Settings.cs" company="">
//   
// </copyright>
// <summary>
//   The settings.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.Main
{
    using System;
    using System.IO;
    using System.Windows.Forms;

    using Globals;

    using HaloMap.Map;

    using Microsoft.Win32;

    // using System.Linq;

    /// <summary>
    /// The settings.
    /// </summary>
    /// <remarks></remarks>
    public partial class Settings : Form
    {
        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Settings"/> class.
        /// </summary>
        /// <remarks></remarks>
        public Settings()
        {
            InitializeComponent();

            textMainmenu.Text = Prefs.pathMainmenu;
            textShared.Text = Prefs.pathShared;
            textSPShared.Text = Prefs.pathSPShared;
            textBitmaps.Text = Prefs.pathBitmaps;
            textMapFolder.Text = Prefs.pathMapFolder;
            textCleanMaps.Text = Prefs.pathCleanMaps;
            textMapFolder.Text = Prefs.pathMapFolder;
            textPluginDir.Text = Prefs.pathPluginsFolder;
            textPatchFolder.Text = Prefs.pathPatchFolder;
            useDefaultMaps.Checked = Prefs.useDefaultMaps;
            checkUpdates.SelectedItem = Prefs.checkUpdate;

            /*
            if (checkUpdates.SelectedIndex == -1)
                checkUpdates.SelectedIndex = 0;
             */
        }

        #endregion

        #region Methods

        /// <summary>
        /// The find directory.
        /// </summary>
        /// <param name="initialDirectory">The initial directory.</param>
        /// <param name="viewName">The view name.</param>
        /// <returns>The find directory.</returns>
        /// <remarks></remarks>
        private string FindDirectory(string initialDirectory, string viewName)
        {
            // Create an OpenFileDialog object.
            FolderBrowserDialog openFolder1 = new FolderBrowserDialog();

            // Initialize the OpenFileDialog to look for text files.
            openFolder1.SelectedPath = initialDirectory;

            // Check if the user selected a file from the OpenFileDialog.
            if (openFolder1.ShowDialog() == DialogResult.OK)
            {
                return openFolder1.SelectedPath;
            }
            else
            {
                return initialDirectory;
            }
        }

        /// <summary>
        /// The find file.
        /// </summary>
        /// <param name="initPath">The init path.</param>
        /// <param name="fileName">The file name.</param>
        /// <param name="viewName">The view name.</param>
        /// <returns>The find file.</returns>
        /// <remarks></remarks>
        private string FindFile(string initPath, string fileName, string viewName)
        {
            if (initPath.LastIndexOf("\\") != -1)
            {
                initPath = initPath.Substring(0, initPath.LastIndexOf("\\"));
            }

            // Create an OpenFileDialog object.
            OpenFileDialog openFile1 = new OpenFileDialog();

            // Initialize the OpenFileDialog to look for text files.
            openFile1.InitialDirectory = initPath;
            openFile1.Filter = viewName + " File|" + fileName;
            openFile1.FileName = fileName;

            // Check if the user selected a file from the OpenFileDialog.
            if (openFile1.ShowDialog() == DialogResult.OK)
            {
                return openFile1.FileName;
            }
            else
            {
                return initPath + "\\" + fileName;
            }
        }

        /// <summary>
        /// The button bitmaps_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonBitmaps_Click(object sender, EventArgs e)
        {
            textBitmaps.Text = FindFile(textBitmaps.Text, "bitmaps.map", "Bitmaps (Only for Halo1 & Halo CE)");
        }

        /// <summary>
        /// The button cancel_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// The button clean maps_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonCleanMaps_Click(object sender, EventArgs e)
        {
            textCleanMaps.Text = FindDirectory(textCleanMaps.Text, "Clean Maps Directory");
        }

        /// <summary>
        /// The button mainmenu_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonMainmenu_Click(object sender, EventArgs e)
        {
            textMainmenu.Text = FindFile(textMainmenu.Text, "mainmenu.map", "MainMenu");
        }

        /// <summary>
        /// The button map folder_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonMapFolder_Click(object sender, EventArgs e)
        {
            textMapFolder.Text = FindDirectory(textMapFolder.Text, "Maps Directory");
        }

        /// <summary>
        /// The button patch folder_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonPatchFolder_Click(object sender, EventArgs e)
        {
            textPatchFolder.Text = FindDirectory(textPatchFolder.Text, "Patches Directory");
        }

        /// <summary>
        /// The button plugin dir_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonPluginDir_Click(object sender, EventArgs e)
        {
            textPluginDir.Text = FindDirectory(textPluginDir.Text, "Plugins Directory");
        }

        /// <summary>
        /// The button sp shared_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonSPShared_Click(object sender, EventArgs e)
        {
            textSPShared.Text = FindFile(textSPShared.Text, "single_player_shared.map", "SP_Shared");
        }

        /// <summary>
        /// The button save_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonSave_Click(object sender, EventArgs e)
        {
            Prefs.pathMainmenu = textMainmenu.Text;
            Prefs.pathShared = textShared.Text;
            Prefs.pathSPShared = textSPShared.Text;
            Prefs.pathBitmaps = textBitmaps.Text;
            Prefs.pathMapFolder = textMapFolder.Text;
            Prefs.pathCleanMaps = textCleanMaps.Text;
            Prefs.pathPluginsFolder = textPluginDir.Text;
            Prefs.pathPatchFolder = textPatchFolder.Text;
            Prefs.useDefaultMaps = useDefaultMaps.Checked;
            Prefs.checkUpdate = (Prefs.updateFrequency)checkUpdates.SelectedItem;

            Prefs.Save();

            this.Close();
        }

        /// <summary>
        /// The button shared_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void buttonShared_Click(object sender, EventArgs e)
        {
            textShared.Text = FindFile(textShared.Text, "shared.map", "Shared");
        }

        #endregion
    }
}