// --------------------------------------------------------------------------------------------------------------------
// <copyright file="BitmapControl.cs" company="">
//   
// </copyright>
// <summary>
//   The bitmap control.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.MapForms
{
    using System;
    using System.Drawing;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;

    using entity.Main;

    using HaloMap;
    using HaloMap.DDSFunctions;
    using HaloMap.Map;
    using HaloMap.Meta;
    using HaloMap.RawData;

    /// <summary>
    /// The bitmap control.
    /// </summary>
    /// <remarks></remarks>
    public partial class BitmapControl : UserControl
    {
        #region Constants and Fields

        /// <summary>
        /// The map number.
        /// </summary>
        public Map map;

        /// <summary>
        /// The pm.
        /// </summary>
        public ParsedBitmap pm;

        /// <summary>
        /// The bitm ptr.
        /// </summary>
        private IntPtr bitmPtr = IntPtr.Zero; // Data pointer for the "editor" bitmap

        /// <summary>
        /// The currbitm.
        /// </summary>
        private int currbitm;

        /// <summary>
        /// The currchunk.
        /// </summary>
        private int currchunk = 0;

        /// <summary>
        /// The currmip.
        /// </summary>
        private int currmip;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BitmapControl"/> class.
        /// </summary>
        /// <param name="map">The map.</param>
        /// <remarks></remarks>
        public BitmapControl(Map map)
        {
            InitializeComponent();
            this.map = map;

            map.OpenMap(MapTypes.Internal);

            pm = new ParsedBitmap(ref map.SelectedMeta, map);

            for (int x = 0; x < pm.Properties.Length; x++)
            {
                TreeNode tn = new TreeNode("Bitmap #" + x);
                tn.Tag = "BITMAP";

                for (int i = 0; i < map.SelectedMeta.raw.rawChunks.Count; i++)
                {
                    int count = pm.Properties[x].mipMapCount;
                    BitmapRawDataChunk bmRaw = (BitmapRawDataChunk)map.SelectedMeta.raw.rawChunks[i];
                    if (bmRaw.inchunk == x)
                    {
                        TreeNode chunknode = new TreeNode("Chunk #" + bmRaw.num);
                        chunknode.Tag = "CHUNK";

                        int chunkWidth = pm.Properties[0].width >> i;
                        int mipcount = 0;
                        while (chunkWidth > 1 && count > 0)
                        {
                            TreeNode mipnode = new TreeNode("Mipmap #" + (mipcount++));
                            mipnode.Tag = "MIPMAP";
                            chunknode.Nodes.Add(mipnode);
                            chunkWidth >>= 1;
                            count--;
                        }

                        tn.Nodes.Add(chunknode);
                    }
                }

                treeView1.Nodes.Add(tn);
            }

            map.CloseMap();

            DisplayBitmap(0, 0, 0);
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// The display bitmap.
        /// </summary>
        /// <param name="bitmap">The bitmap.</param>
        /// <param name="chunk">The chunk.</param>
        /// <param name="mipmap">The mipmap.</param>
        /// <remarks></remarks>
        public void DisplayBitmap(int bitmap, int chunk, int mipmap)
        {
            if (bitmPtr != IntPtr.Zero)
            {
                Marshal.FreeHGlobal(bitmPtr);
            }

            pictureBox1.Image = pm.FindChunkAndDecode(
                bitmap, chunk, mipmap, ref map.SelectedMeta, map, chunk, 0);

            ParsedBitmap.BitmapInfo bmInfo = new ParsedBitmap.BitmapInfo(
                pm.Properties[0].formatname, pm.Properties[0].swizzle);

            ////////////// Remove this. For testing Decoding/Encoding ///////////////
            // if (!pm.Properties[0].formatname.ToString().ToUpper().Contains("DXT"))
            // pictureBox1.Image = DDS_Convert.DecodeDDS(DDS_Convert.EncodeDDS((Bitmap)pictureBox1.Image, ref bmInfo), bmInfo);
            pictureBox1.Width = (pictureBox1.Image.Width * trackBar1.Value) / 2;
            pictureBox1.Height = (pictureBox1.Image.Height * trackBar1.Value) / 2;
            pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
            currbitm = chunk;
            currmip = mipmap;
        }

        /// <summary>
        /// The refresh.
        /// </summary>
        /// <remarks></remarks>
        public void RefreshDisplay()
        {
            DisplayBitmap(currbitm, currchunk, currmip);
        }

        #endregion

        #region Methods

        /// <summary>
        /// The panel 1_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void panel1_Click(object sender, EventArgs e)
        {
            switch (this.BackColor.Name)
            {
                case "Black":
                    if (((MouseEventArgs)e).Button == MouseButtons.Left)
                    {
                        this.BackColor = Color.White;
                    }
                    else
                    {
                        this.BackColor = Color.Gray;
                    }

                    break;
                case "White":
                    if (((MouseEventArgs)e).Button == MouseButtons.Left)
                    {
                        this.BackColor = Color.Red;
                    }
                    else
                    {
                        this.BackColor = Color.Black;
                    }

                    break;
                case "Red":
                    if (((MouseEventArgs)e).Button == MouseButtons.Left)
                    {
                        this.BackColor = Color.Blue;
                    }
                    else
                    {
                        this.BackColor = Color.White;
                    }

                    break;
                case "Blue":
                    if (((MouseEventArgs)e).Button == MouseButtons.Left)
                    {
                        this.BackColor = Color.Yellow;
                    }
                    else
                    {
                        this.BackColor = Color.Red;
                    }

                    break;
                case "Yellow":
                    if (((MouseEventArgs)e).Button == MouseButtons.Left)
                    {
                        this.BackColor = Color.SteelBlue;
                    }
                    else
                    {
                        this.BackColor = Color.Blue;
                    }

                    break;
                case "SteelBlue":
                    if (((MouseEventArgs)e).Button == MouseButtons.Left)
                    {
                        this.BackColor = Color.Gray;
                    }
                    else
                    {
                        this.BackColor = Color.Yellow;
                    }

                    break;
                case "Gray":
                    if (((MouseEventArgs)e).Button == MouseButtons.Left)
                    {
                        this.BackColor = Color.Black;
                    }
                    else
                    {
                        this.BackColor = Color.SteelBlue;
                    }

                    break;
                default:
                    this.BackColor = Color.Black;
                    break;
            }
        }

        /// <summary>
        /// The picture box 1_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void pictureBox1_Click(object sender, EventArgs e)
        {
            // Use a temp or it can get an oversize error.
            int temp = trackBar1.Value;
            temp += 2;
            if (temp > 6)
            {
                temp = 2;
            }

            trackBar1.Value = temp;
            DisplayBitmap(currbitm, currchunk, currmip);
        }

        /// <summary>
        /// The track bar 1_ scroll.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            this.label1.Text = (trackBar1.Value / 2.0f).ToString("0.0'x'");
            DisplayBitmap(currbitm, currchunk, currmip);
        }

        /// <summary>
        /// The tree view 1_ after select.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            int index = 0;
            int chunk = 0;
            int mipmap = 0;
            if (((string)treeView1.SelectedNode.Tag) == "MIPMAP")
            {
                index = treeView1.SelectedNode.Parent.Parent.Index;
                chunk = treeView1.SelectedNode.Parent.Index;
                mipmap = treeView1.SelectedNode.Index;
            }
            else if (((string)treeView1.SelectedNode.Tag) == "CHUNK")
            {
                index = treeView1.SelectedNode.Parent.Index;
                chunk = treeView1.SelectedNode.Index;
            }
            else
            {
                index = treeView1.SelectedNode.Index;
            }

            DisplayBitmap(index, chunk, mipmap);

            // Wow. That's alot of parents. o SHOULD be type MapForm
            // object o = this.Parent.Parent.Parent.Parent.Parent.Parent;
            object o = ((Form1)this.TopLevelControl).ActiveMdiChild;
            if (o is MapForm)
            {
                MapForm mf = (MapForm)o;
                Meta meta = map.SelectedMeta;
                ParsedBitmap pm = new ParsedBitmap(ref meta, map);
                //if (mf.bitmMainPtr != IntPtr.Zero)
                //{
                //    Marshal.FreeHGlobal(mf.bitmMainPtr);
                //}

                Bitmap b = pm.FindChunkAndDecode(index, chunk, mipmap, ref meta, map, 0, 0);
                mf.pictureBox = (Bitmap)pictureBox1.Image;
                mf.statusBarText = (pm.Properties[index].width >> chunk >> mipmap).ToString().PadLeft(4) + " X " +
                                   (pm.Properties[index].height >> chunk >> mipmap).ToString().PadRight(4) + "X " +
                                   (pm.Properties[index].depth >> chunk >> mipmap).ToString().PadRight(4) + " " +
                                   ("(" + pm.Properties[index].typename.ToString().Remove(0, 10) + ") ").PadRight(10) +
                                   pm.Properties[index].formatname.ToString().Remove(0, 12).PadRight(10) + " - Swizzle:" +
                                   pm.Properties[0].swizzle + "- Location: " + meta.raw.rawChunks[0].rawLocation;
            }
        }

        #endregion
    }
}