// --------------------------------------------------------------------------------------------------------------------
// <copyright file="DataValues.cs" company="">
//   
// </copyright>
// <summary>
//   The data values.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.MetaEditor2
{
    using System;
    using System.IO;
    using System.Windows.Forms;

    using HaloMap.Map;
    using HaloMap.Plugins;
    using HaloMap.RealTimeHalo;

    /// <summary>
    /// The data values.
    /// </summary>
    /// <remarks></remarks>
    public partial class DataValues : BaseField
    {
        // public int LineNumber;
        // private int mapIndex;
        // public int chunkOffset;
        // public int offsetInMap;
        // public string EntName = "Error in getting plugin element name";
        #region Constants and Fields

        /// <summary>
        /// The value.
        /// </summary>
        public object Value;

        /// <summary>
        /// The value type.
        /// </summary>
        public IFPIO.ObjectEnum ValueType;

        /// <summary>
        /// The is nulled out reflexive.
        /// </summary>
        private bool isNulledOutReflexive = true;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DataValues"/> class.
        /// </summary>
        /// <param name="iEntName">Name of the i ent.</param>
        /// <param name="map">The map.</param>
        /// <param name="iOffsetInChunk">The i offset in chunk.</param>
        /// <param name="type">The type.</param>
        /// <param name="iLineNumber">The i line number.</param>
        /// <remarks></remarks>
        public DataValues(string iEntName, Map map, int iOffsetInChunk, IFPIO.ObjectEnum type, int iLineNumber)
        {
            this.LineNumber = iLineNumber;
            this.chunkOffset = iOffsetInChunk;
            mapIndex = map;
            this.EntName = iEntName;
            InitializeComponent();
            this.Dock = DockStyle.Top;
            this.Controls[0].Text = EntName;
            this.Controls[2].Text = type.ToString();
            this.ValueType = type;
            this.AutoSize = false;
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// The base field_ leave.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <exception cref="NotImplementedException">
        ///   </exception>
        /// <remarks></remarks>
        public override void BaseField_Leave(object sender, EventArgs e)
        {
            BinaryWriter bw = new BinaryWriter(mapIndex.SelectedMeta.MS);
            if (((WinMetaEditor)this.ParentForm).checkSelectionInCurrentTag())
            {
                bw.BaseStream.Position = this.offsetInMap - mapIndex.SelectedMeta.offset;
            }

            /*
            if (((WinMetaEditor)this.ParentForm).checkSelectionInCurrentTag())
            else
            {
                mapIndex.OpenMap(MapTypes.Internal);
                bw = mapIndex.BW;
                mapIndex.SelectedMeta.MS.Position = this.offsetInMap;
                // ...
                mapIndex.CloseMap();
            }
            */
            switch (this.ValueType)
            {
                case IFPIO.ObjectEnum.Byte:
                    bw.Write(byte.Parse(this.Value.ToString()));
                    break;
                case IFPIO.ObjectEnum.Int:
                    bw.Write(int.Parse(this.Value.ToString()));
                    break;
                case IFPIO.ObjectEnum.Short:
                    bw.Write(short.Parse(this.Value.ToString()));
                    break;
                case IFPIO.ObjectEnum.UInt:
                    bw.Write(uint.Parse(this.Value.ToString()));
                    break;
                case IFPIO.ObjectEnum.UShort:
                    bw.Write(ushort.Parse(this.Value.ToString()));
                    break;
                case IFPIO.ObjectEnum.Float:
                case IFPIO.ObjectEnum.Unknown:
                    bw.Write(float.Parse(this.Value.ToString()));
                    break;
                case IFPIO.ObjectEnum.Unused:
                    break;
                default:
                    throw new NotImplementedException("Unknown Type While Saving: " + this.ValueType);
            }
        }

        /// <summary>
        /// The poke.
        /// </summary>
        /// <remarks></remarks>
        public void Poke()
        {
            uint Address = (uint)(this.offsetInMap + mapIndex.SelectedMeta.magic);
            switch (ValueType)
            {
                case IFPIO.ObjectEnum.Short:
                    {
                        RTH_Imports.Poke(Address, (uint)Convert.ToInt16(this.Value), 16);
                        break;
                    }

                case IFPIO.ObjectEnum.Int:
                    {
                        RTH_Imports.Poke(Address, (uint)Convert.ToInt32(this.Value), 32);
                        break;
                    }

                case IFPIO.ObjectEnum.UShort:
                    {
                        RTH_Imports.Poke(Address, Convert.ToUInt16(this.Value), 16);
                        break;
                    }

                case IFPIO.ObjectEnum.UInt:
                    {
                        RTH_Imports.Poke(Address, Convert.ToUInt32(this.Value), 32);
                        break;
                    }

                case IFPIO.ObjectEnum.Float:
                    {
                        RTH_Imports.Poke(Address, RTH_Imports.ConvertFloat(Convert.ToSingle(this.Value)), 32);
                        break;
                    }

                case IFPIO.ObjectEnum.Unknown:
                    {
                        RTH_Imports.Poke(Address, RTH_Imports.ConvertFloat(Convert.ToSingle(this.Value)), 32);
                        break;
                    }

                case IFPIO.ObjectEnum.Byte:
                    {
                        RTH_Imports.Poke(Address, Convert.ToByte(this.Value), 8);
                        break;
                    }
            }
        }

        /// <summary>
        /// The populate.
        /// </summary>
        /// <param name="iOffset">The i offset.</param>
        /// <param name="useMemoryStream">The use memory stream.</param>
        /// <remarks></remarks>
        public void Populate(int iOffset, bool useMemoryStream)
        {
            this.isNulledOutReflexive = false;
            BinaryReader BR = new BinaryReader(mapIndex.SelectedMeta.MS);

            // set offsets
            BR.BaseStream.Position = iOffset + this.chunkOffset;
            this.offsetInMap = iOffset + this.chunkOffset;

            // If we need to read / save tag info directly to file...
            if (!useMemoryStream)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                BR = mapIndex.BR;
                BR.BaseStream.Position = this.offsetInMap;
            }
            else
            {
                this.offsetInMap += mapIndex.SelectedMeta.offset;
            }

            switch (ValueType)
            {
                case IFPIO.ObjectEnum.Short:
                    {
                        this.Value = BR.ReadInt16();
                        break;
                    }

                case IFPIO.ObjectEnum.Int:
                    {
                        this.Value = BR.ReadInt32();
                        break;
                    }

                case IFPIO.ObjectEnum.UShort:
                    {
                        this.Value = BR.ReadUInt16();
                        break;
                    }

                case IFPIO.ObjectEnum.UInt:
                    {
                        this.Value = BR.ReadUInt32();
                        break;
                    }

                case IFPIO.ObjectEnum.Float:
                    {
                        this.Value = BR.ReadSingle();
                        break;
                    }

                case IFPIO.ObjectEnum.Unknown:
                    {
                        this.Value = BR.ReadSingle();
                        break;
                    }

                case IFPIO.ObjectEnum.Byte:
                    {
                        this.Value = BR.ReadByte();
                        break;
                    }
            }

            // ...and then close the file once we are done!
            if (!useMemoryStream)
            {
                mapIndex.CloseMap();
            }

            if (this.ValueType != IFPIO.ObjectEnum.Unused)
            {
                this.Controls[1].Text = this.Value.ToString();
            }
        }

        /// <summary>
        /// The save.
        /// </summary>
        /// <exception cref="NotImplementedException">
        ///   </exception>
        /// <remarks></remarks>
        public override void Save()
        {
            if (this.isNulledOutReflexive)
            {
                return;
            }

            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }

            try
            {
                mapIndex.BW.BaseStream.Position = this.offsetInMap;
                switch (ValueType)
                {
                    case IFPIO.ObjectEnum.Short:
                        {
                            mapIndex.BW.Write(Convert.ToInt16(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Int:
                        {
                            mapIndex.BW.Write(Convert.ToInt32(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.UShort:
                        {
                            mapIndex.BW.Write(Convert.ToUInt16(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.UInt:
                        {
                            mapIndex.BW.Write(Convert.ToUInt32(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Float:
                        {
                            mapIndex.BW.Write(Convert.ToSingle(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Unknown:
                        {
                            mapIndex.BW.Write(Convert.ToSingle(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Byte:
                        {
                            mapIndex.BW.Write(Convert.ToByte(this.Value));
                            break;
                        }
                }
            }
            catch //(Exception e)
            {
                MessageBox.Show(
                    "Something is wrong with this " + this.ValueType + this.EntName + " Offset " + this.chunkOffset);
            }

            if (openedMap)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The set focus.
        /// </summary>
        /// <param name="LineToCheck">The line to check.</param>
        /// <remarks></remarks>
        public void SetFocus(int LineToCheck)
        {
            if (this.LineNumber == LineToCheck)
            {
                this.Focus();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// The text box 1_ text changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            this.Value = ((TextBox)sender).Text;
        }

        #endregion
    }
}