// --------------------------------------------------------------------------------------------------------------------
// <copyright file="EntStrings.cs" company="">
//   
// </copyright>
// <summary>
//   The ent strings.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.MetaEditor2
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Text;
    using System.Windows.Forms;

    using HaloMap.Map;

    /// <summary>
    /// The ent strings.
    /// </summary>
    /// <remarks></remarks>
    public partial class EntStrings : BaseField
    {
        #region Constants and Fields

        /// <summary>
        /// The ent unicode.
        /// </summary>
        private readonly bool entUnicode;

        /// <summary>
        /// The length.
        /// </summary>
        private readonly int length;

        /// <summary>
        /// The is nulled out reflexive.
        /// </summary>
        private bool isNulledOutReflexive = true;

        // public int LineNumber;
        // private int mapIndex;
        // public int chunkOffset;
        // private int offsetInMap;
        // public string EntName = "Error in getting plugin element name";
        /// <summary>
        /// The value.
        /// </summary>
        private string value = string.Empty;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="EntStrings"/> class.
        /// </summary>
        /// <param name="iEntName">Name of the i ent.</param>
        /// <param name="map">The map.</param>
        /// <param name="iOffsetInChunk">The i offset in chunk.</param>
        /// <param name="ilength">The ilength.</param>
        /// <param name="itype">if set to <c>true</c> [itype].</param>
        /// <param name="iLineNumber">The i line number.</param>
        /// <remarks></remarks>
        public EntStrings(string iEntName, Map map, int iOffsetInChunk, int ilength, bool itype, int iLineNumber)
        {
            this.LineNumber = iLineNumber;
            this.entUnicode = itype;
            this.length = ilength;
            mapIndex = map;
            this.EntName = iEntName;
            this.chunkOffset = iOffsetInChunk;
            InitializeComponent();
            this.label2.Text = EntName;
            this.Dock = DockStyle.Top;
            this.Size = this.PreferredSize;
            this.AutoSize = false;
            this.Leave += String_Leave;
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// The populate.
        /// </summary>
        /// <param name="iOffset">The i offset.</param>
        /// <param name="useMemoryStream">The use memory stream.</param>
        /// <remarks></remarks>
        public void Populate(int iOffset, bool useMemoryStream)
        {
            this.isNulledOutReflexive = false;
            BinaryReader BR = new BinaryReader(mapIndex.SelectedMeta.MS);

            // set offsets
            BR.BaseStream.Position = iOffset + this.chunkOffset;
            this.offsetInMap = iOffset + this.chunkOffset;

            // If we need to read / save tag info directly to file...
            if (!useMemoryStream)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                BR = mapIndex.BR;
                BR.BaseStream.Position = this.offsetInMap;
            }
            else
            {
                this.offsetInMap += mapIndex.SelectedMeta.offset;
            }

            if (this.entUnicode)
            {
                Encoding decode = Encoding.Unicode;
                byte[] tempbytes = BR.ReadBytes(this.length);
                this.value = decode.GetString(tempbytes);
            }
            else
            {
                this.value = new string(BR.ReadChars(this.length));
            }

            this.value = this.value.TrimEnd('\0');

            // this.RemoveNullCharFromValue();
            this.textBox1.Text = this.value;
            this.textBox1.Size = this.textBox1.PreferredSize;
            this.textBox1.Width += 5;

            // ...and then close the file once we are done!
            if (!useMemoryStream)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The save.
        /// </summary>
        /// <exception cref="NotImplementedException">
        ///   </exception>
        /// <remarks></remarks>
        public override void Save()
        {
            if (this.isNulledOutReflexive)
            {
                return;
            }

            // Why doesn't it allow saving of unicode Strings??
            if (this.entUnicode)
            {
                return;
            }

            if (this.value == "e1_mars_entry12")
            {
                this.value = "e1_mars_entry12";
            }

            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }

            mapIndex.BW.BaseStream.Position = this.offsetInMap;
            for (int counter = 0; counter < this.length / 4; counter++)
            {
                mapIndex.BW.Write(0);
            }

            mapIndex.BW.BaseStream.Position = this.offsetInMap;

            // if (this.entUnicode == true)
            // {
            // for (int counter = 0; counter < this.value.Length; counter++)
            // {
            // mapIndex.BW.Write((char)this.value[counter]);
            // if (mapIndex.BW.BaseStream.Position < (long)this.offsetInMap + 2 * (counter+1))
            // mapIndex.BW.BaseStream.Position++;
            // }
            // }
            // else
            // {
            for (int counter = 0; counter < this.value.Length; counter++)
            {
                mapIndex.BW.Write(this.value[counter]);
            }

            // }
            if (openedMap)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The set focus.
        /// </summary>
        /// <param name="LineToCheck">The line to check.</param>
        /// <remarks></remarks>
        public void SetFocus(int LineToCheck)
        {
            if (this.LineNumber == LineToCheck)
            {
                this.Focus();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// The remove null char from value.
        /// </summary>
        /// <remarks></remarks>
        private void RemoveNullCharFromValue()
        {
            List<char> tempchar = new List<char>(0);
            tempchar.AddRange(this.value.ToCharArray());
            for (int counter = 0; counter < tempchar.Count; counter++)
            {
                if (tempchar[counter] == '\0')
                {
                    tempchar.RemoveAt(counter);
                    counter--;
                }
            }

            this.value = new string(tempchar.ToArray());
        }

        /// <summary>
        /// The string_ leave.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void String_Leave(object sender, EventArgs e)
        {
            // if (this.AutoSave)
            // this.Save();
        }

        /// <summary>
        /// The text box 1_ text changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            this.textBox1.Size = this.textBox1.PreferredSize;

            // this.label2.Location = new Point(((TextBox)sender).Location.X + ((TextBox)sender).PreferredSize.Width + 5,6);
            this.value = ((TextBox)sender).Text;
        }

        #endregion
    }
}