// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Enums.cs" company="">
//   
// </copyright>
// <summary>
//   The enums.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.MetaEditor2
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Windows.Forms;

    using HaloMap.Map;
    using HaloMap.Plugins;
    using HaloMap.RealTimeHalo;

    /// <summary>
    /// The enums.
    /// </summary>
    /// <remarks></remarks>
    public partial class Enums : BaseField
    {
        #region Constants and Fields

        /// <summary>
        /// The options.
        /// </summary>
        public object[] Options;

        /// <summary>
        /// The enum type.
        /// </summary>
        public int enumType;

        // public int LineNumber;
        // private int mapIndex;
        // public int chunkOffset;
        // public int offsetInMap;
        // public string EntName = "Error in getting plugin element name";

        /// <summary>
        /// The value.
        /// </summary>
        public int value;

        /// <summary>
        /// The is nulled out reflexive.
        /// </summary>
        private bool isNulledOutReflexive = true;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Enums"/> class.
        /// </summary>
        /// <param name="iEntName">Name of the i ent.</param>
        /// <param name="map">The map.</param>
        /// <param name="iOffsetInChunk">The i offset in chunk.</param>
        /// <param name="iType">Type of the i.</param>
        /// <param name="ioptions">The ioptions.</param>
        /// <param name="iLineNumber">The i line number.</param>
        /// <remarks></remarks>
        public Enums(string iEntName, Map map, int iOffsetInChunk, int iType, object[] ioptions, int iLineNumber)
        {
            this.LineNumber = iLineNumber;
            this.Options = ioptions;
            this.enumType = iType;
            this.chunkOffset = iOffsetInChunk;
            mapIndex = map;
            this.EntName = iEntName;
            InitializeComponent();
            this.Size = this.PreferredSize;
            this.Dock = DockStyle.Top;
            this.Controls[0].Text = EntName;
            switch (iType)
            {
                case 8:
                    {
                        this.Controls[2].Text = "Enum8";
                        break;
                    }

                case 16:
                    {
                        this.Controls[2].Text = "Enum16";
                        break;
                    }

                case 32:
                    {
                        this.Controls[2].Text = "Enum32";
                        break;
                    }
            }

            this.AutoSize = false;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets Value.
        /// </summary>
        /// <remarks></remarks>
        public string Value
        {
            get
            {
                return this.value.ToString();
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// The base field_ leave.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        public override void BaseField_Leave(object sender, EventArgs e)
        {
            // if (this.AutoSave)
            // this.Save();
        }

        /// <summary>
        /// The poke.
        /// </summary>
        /// <remarks></remarks>
        public void Poke()
        {
            uint Address = (uint)(this.offsetInMap + mapIndex.SelectedMeta.magic);
            this.UpdateValue();
            switch (this.enumType)
            {
                case 8:
                    {
                        RTH_Imports.Poke(Address, Convert.ToByte(this.value), 8);
                        break;
                    }

                case 16:
                    {
                        RTH_Imports.Poke(Address, (uint)Convert.ToInt16(this.value), 16);
                        break;
                    }

                case 32:
                    {
                        RTH_Imports.Poke(Address, (uint)this.value, 32);
                        break;
                    }
            }
        }

        /// <summary>
        /// The populate.
        /// </summary>
        /// <param name="iOffset">The i offset.</param>
        /// <param name="useMemoryStream">The use memory stream.</param>
        /// <remarks></remarks>
        public void Populate(int iOffset, bool useMemoryStream)
        {
            this.isNulledOutReflexive = false;
            BinaryReader BR = new BinaryReader(mapIndex.SelectedMeta.MS);

            // set offsets
            BR.BaseStream.Position = iOffset + this.chunkOffset;
            this.offsetInMap = iOffset + this.chunkOffset;

            // If we need to read / save tag info directly to file...
            if (!useMemoryStream)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                BR = mapIndex.BR;
                BR.BaseStream.Position = this.offsetInMap;
            }
            else
            {
                this.offsetInMap += mapIndex.SelectedMeta.offset;
            }

            switch (this.enumType)
            {
                case 8:
                    {
                        this.value = BR.ReadByte();
                        break;
                    }

                case 16:
                    {
                        this.value = BR.ReadInt16();
                        break;
                    }

                case 32:
                    {
                        this.value = BR.ReadInt32();
                        break;
                    }
            }

            // ...and then close the file once we are done!
            if (!useMemoryStream)
            {
                mapIndex.CloseMap();
            }

            UpdateComboBox();
        }

        /// <summary>
        /// The save.
        /// </summary>
        /// <exception cref="NotImplementedException">
        ///   </exception>
        /// <remarks></remarks>
        public override void Save()
        {
            if (this.isNulledOutReflexive)
            {
                return;
            }

            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }

            this.UpdateValue();
            mapIndex.BW.BaseStream.Position = this.offsetInMap;
            try
            {
                switch (this.enumType)
                {
                    case 8:
                        {
                            mapIndex.BW.Write(Convert.ToByte(this.value));
                            break;
                        }

                    case 16:
                        {
                            mapIndex.BW.Write(Convert.ToInt16(this.value));
                            break;
                        }

                    case 32:
                        {
                            mapIndex.BW.Write(this.value);
                            break;
                        }
                }
            }
            catch //(Exception e)
            {
                MessageBox.Show("Something is wrong with this enum " + this.EntName + " Offset " + this.chunkOffset);
            }

            if (openedMap)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The set focus.
        /// </summary>
        /// <param name="LineToCheck">The line to check.</param>
        /// <remarks></remarks>
        public void SetFocus(int LineToCheck)
        {
            if (this.LineNumber == LineToCheck)
            {
                this.Focus();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// The update combo box.
        /// </summary>
        /// <remarks></remarks>
        private void UpdateComboBox()
        {
            // add all the values from the options list to a temp list<string>.
            List<int> tempint = new List<int>(0);
            if (this.Options != null)
            {
                for (int counter = 0; counter < this.Options.Length; counter++)
                {
                    tempint.Add(((IFPIO.Option)this.Options[counter]).value);
                }
            }

            // see if the value in the map is in the list, if not, add it.
            if (tempint.IndexOf(this.value) == -1)
            {
                tempint.Add(this.value);
            }

            tempint.TrimExcess();
            tempint.Sort();
            int tempindex = tempint.IndexOf(this.value);

            // go through and put the option name in the strings after the ": "
            List<string> tempstring = new List<string>(0);
            for (int counter = 0; counter < tempint.Count; counter++)
            {
                tempstring.Add(tempint[counter].ToString());
            }

            if (this.Options != null)
            {
                for (int counter = 0; counter < this.Options.Length; counter++)
                {
                    if (tempint.IndexOf(((IFPIO.Option)this.Options[counter]).value) != -1)
                    {
                        tempstring[tempint.IndexOf(((IFPIO.Option)this.Options[counter]).value)] += ": " +
                                                                                                    ((IFPIO.Option)
                                                                                                     this.Options[
                                                                                                         counter]).name;
                    }
                }
            }

            this.comboBox1.Items.Clear();
            this.comboBox1.Items.AddRange(tempstring.ToArray());
            this.comboBox1.SelectedIndex = tempindex;
        }

        /// <summary>
        /// The update value.
        /// </summary>
        /// <remarks></remarks>
        private void UpdateValue()
        {
            // Since I mixed the name with the value, we have to take out the names.
            string tempstring = this.comboBox1.Text;
            for (int counter = 0; counter < tempstring.Length; counter++)
            {
                if (tempstring[counter] == ':')
                {
                    tempstring = tempstring.Substring(0, counter);
                    break;
                }
            }

            // if some n00b (read xbox7887) decided to type something stupid into the enum box, here we just test to see if it'll work.
            // if not, the value stays the same as what was read in
            try
            {
                this.value = Convert.ToInt32(tempstring);
            }
            catch
            {
            }
        }

        #endregion
    }
}