// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Ident.cs" company="">
//   
// </copyright>
// <summary>
//   The ident.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.MetaEditor2
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.IO;
    using System.Windows.Forms;

    using HaloMap.Map;
    using HaloMap.RealTimeHalo;

    /// <summary>
    /// The ident.
    /// </summary>
    /// <remarks></remarks>
    public partial class Ident : BaseField
    {
        // public int LineNumber;
        // private int mapIndex;
        // public int chunkOffset;
        // public int offsetInMap;
        // public string EntName = "Error in getting plugin element name";
        #region Constants and Fields

        /// <summary>
        /// The ident int 32.
        /// </summary>
        public int identInt32;

        /// <summary>
        /// The tag index.
        /// </summary>
        public int tagIndex;

        /// <summary>
        /// The tag name.
        /// </summary>
        public string tagName;

        /// <summary>
        /// The tag type.
        /// </summary>
        public string tagType;

        /// <summary>
        /// The does have tag type.
        /// </summary>
        private readonly bool doesHaveTagType;

        /// <summary>
        /// The add events.
        /// </summary>
        private bool AddEvents = true;

        /// <summary>
        /// The is nulled out reflexive.
        /// </summary>
        private bool isNulledOutReflexive = true;

        /// <summary>
        /// The tag combo box indexer.
        /// </summary>
        private int tagComboBoxIndexer;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Ident"/> class.
        /// </summary>
        /// <param name="iEntName">Name of the i ent.</param>
        /// <param name="map">The map.</param>
        /// <param name="iOffsetInChunk">The i offset in chunk.</param>
        /// <param name="idoesHaveTagType">if set to <c>true</c> [idoes have tag type].</param>
        /// <param name="iLineNumber">The i line number.</param>
        /// <remarks></remarks>
        public Ident(string iEntName, Map map, int iOffsetInChunk, bool idoesHaveTagType, int iLineNumber)
        {
            this.LineNumber = iLineNumber;
            this.doesHaveTagType = idoesHaveTagType;
            mapIndex = map;
            this.EntName = iEntName;
            this.chunkOffset = iOffsetInChunk;
            InitializeComponent();
            this.Controls[0].Text = EntName;
            this.Dock = DockStyle.Top;
            this.Size = this.PreferredSize;
            this.AutoSize = false;
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// The base field_ leave.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        public override void BaseField_Leave(object sender, EventArgs e)
        {
            // if (this.AutoSave)
            // this.Save();
        }

        /// <summary>
        /// The poke.
        /// </summary>
        /// <remarks></remarks>
        public void Poke()
        {
            uint Address = (uint)(this.offsetInMap + mapIndex.SelectedMeta.magic);
            this.FindByNameAndTagType();
            RTH_Imports.Poke(Address, (uint)this.identInt32, 32);
        }

        /// <summary>
        /// The populate.
        /// </summary>
        /// <param name="iOffset">The i offset.</param>
        /// <param name="useMemoryStream">The use memory stream.</param>
        /// <remarks></remarks>
        public void Populate(int iOffset, bool useMemoryStream)
        {
            this.isNulledOutReflexive = false;
            BinaryReader BR = new BinaryReader(mapIndex.SelectedMeta.MS);

            // set offsets
            BR.BaseStream.Position = iOffset + this.chunkOffset;
            this.offsetInMap = iOffset + this.chunkOffset;

            // If we need to read / save tag info directly to file...
            if (!useMemoryStream)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                BR = mapIndex.BR;
                BR.BaseStream.Position = this.offsetInMap;
            }
            else
            {
                this.offsetInMap += mapIndex.SelectedMeta.offset;
            }

            int tempint = BR.ReadInt32();

            // ...and then close the file once we are done!
            if (!useMemoryStream)
            {
                mapIndex.CloseMap();
            }

            this.tagIndex = mapIndex.Functions.ForMeta.FindMetaByID(tempint);
            if (this.tagIndex != -1)
            {
                this.tagName = mapIndex.FileNames.Name[this.tagIndex];
                this.tagType = mapIndex.MetaInfo.TagType[this.tagIndex];
            }
            else
            {
                this.tagName = "null";
                this.tagType = "null";
            }

            this.identInt32 = tempint;
            this.comboBox1.Size = this.comboBox1.PreferredSize;
            if (AddEvents)
            {
                AddEvents = false;
                this.comboBox1.MouseClick += this.comboBox1_MouseClick;
                this.comboBox1.DropDownClosed += this.TagName_DropDownClose;
                this.comboBox2.SelectedIndexChanged += this.TagType_SelectedIndexChanged;
                this.comboBox2.DropDown += this.TagType_DropDown;
                this.comboBox1.TextChanged += this.comboBox1_TextChanged;
            }

            this.UpdateIdentBoxes();
        }

        /// <summary>
        /// The save.
        /// </summary>
        /// <exception cref="NotImplementedException">
        ///   </exception>
        /// <remarks></remarks>
        public override void Save()
        {
            if (this.isNulledOutReflexive)
            {
                return;
            }

            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }

            this.FindByNameAndTagType();
            if (this.doesHaveTagType)
            {
                mapIndex.BW.BaseStream.Position = this.offsetInMap - 4;
                if (this.tagType != "null")
                {
                    List<char> tempList = new List<char>(0);
                    tempList.AddRange(this.tagType.ToCharArray(0, 4));
                    tempList.TrimExcess();
                    tempList.Reverse();
                    char[] tempchar = tempList.ToArray();
                    mapIndex.BW.Write(tempchar);
                }
            }

            mapIndex.BW.BaseStream.Position = this.offsetInMap;
            mapIndex.BW.Write(this.identInt32);
            if (openedMap)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The set focus.
        /// </summary>
        /// <param name="LineToCheck">The line to check.</param>
        /// <remarks></remarks>
        public void SetFocus(int LineToCheck)
        {
            if (this.LineNumber == LineToCheck)
            {
                this.Focus();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// The fill tag box with names.
        /// </summary>
        /// <param name="changeTagType">The change tag type.</param>
        /// <remarks></remarks>
        private void FillTagBoxWithNames(bool changeTagType)
        {
            this.tagType = this.comboBox2.Text;
            this.comboBox1.Items.Clear();
            this.comboBox1.Sorted = true;
            if (this.Controls[2].Text != "null")
            {
                for (int counter = 0; counter < mapIndex.FileNames.Name.Length; counter++)
                {
                    if (this.tagType == mapIndex.MetaInfo.TagType[counter])
                    {
                        this.comboBox1.Items.Add(mapIndex.FileNames.Name[counter]);
                    }
                }
            }

            this.comboBox1.Sorted = false;
            this.comboBox1.Items.Add("null");
            if (changeTagType)
            {
                this.comboBox1.SelectedIndex = 0;
                this.tagComboBoxIndexer = 0;
                this.tagName = (string)this.comboBox1.Items[0];
            }
            else
            {
                this.tagComboBoxIndexer = this.comboBox1.Items.IndexOf(tagName);
            }
        }

        /// <summary>
        /// The find by name and tag type.
        /// </summary>
        /// <remarks></remarks>
        private void FindByNameAndTagType()
        {
            this.tagName = this.comboBox1.Text;
            this.tagType = this.comboBox2.Text;
            for (int x = 0; x < mapIndex.IndexHeader.metaCount; x++)
            {
                if (this.tagType == mapIndex.MetaInfo.TagType[x] && this.tagName == mapIndex.FileNames.Name[x])
                {
                    this.identInt32 = mapIndex.MetaInfo.Ident[x];
                    return;
                }
            }

            this.identInt32 = -1;
        }

        /// <summary>
        /// The tag name_ drop down close.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void TagName_DropDownClose(object sender, EventArgs e)
        {
            if (((ComboBox)sender).Items[((ComboBox)sender).SelectedIndex].ToString() == "null")
            {
                this.comboBox2.Text = "null";
            }
        }

        /// <summary>
        /// The tag type_ drop down.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void TagType_DropDown(object sender, EventArgs e)
        {
            ((ComboBox)sender).Items.Clear();
            IEnumerator i = mapIndex.MetaInfo.TagTypes.Keys.GetEnumerator();
            while (i.MoveNext())
            {
                ((ComboBox)sender).Items.Add(i.Current);
            }

            ((ComboBox)sender).Sorted = true;
            ((ComboBox)sender).Sorted = false;
            ((ComboBox)sender).Items.Add("null");
        }

        /// <summary>
        /// The tag type_ selected index changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void TagType_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.comboBox1.MouseClick -= this.comboBox1_MouseClick;
            this.FillTagBoxWithNames(true);
        }

        /// <summary>
        /// The update ident boxes.
        /// </summary>
        /// <remarks></remarks>
        private void UpdateIdentBoxes()
        {
            this.comboBox1.Text = this.tagName;
            this.comboBox2.Text = this.tagType;
        }

        /// <summary>
        /// The combo box 1_ mouse click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void comboBox1_MouseClick(object sender, MouseEventArgs e)
        {
            this.FillTagBoxWithNames(false);
            ((ComboBox)sender).MouseClick -= this.comboBox1_MouseClick;
        }

        /// <summary>
        /// The combo box 1_ text changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void comboBox1_TextChanged(object sender, EventArgs e)
        {
            ((ComboBox)sender).Width = (((ComboBox)sender).Text.Length * 5) + 50;
        }

        #endregion
    }
}