// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Indices.cs" company="">
//   
// </copyright>
// <summary>
//   The indices.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.MetaEditor2
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Text;
    using System.Windows.Forms;

    using Globals;

    using HaloMap.Map;
    using HaloMap.Plugins;
    using HaloMap.RealTimeHalo;

    /// <summary>
    /// The indices.
    /// </summary>
    /// <remarks></remarks>
    public partial class Indices : BaseField
    {
        // public int LineNumber;
        // private int mapIndex;
        // public int chunkOffset;
        // public int offsetInMap;
        // public string EntName = "Error in getting plugin element name";
        #region Constants and Fields

        /// <summary>
        /// The value.
        /// </summary>
        public int Value;

        /// <summary>
        /// The value type.
        /// </summary>
        public IFPIO.ObjectEnum ValueType;

        /// <summary>
        /// The ent index.
        /// </summary>
        private readonly IFPIO.Index EntIndex;

        /// <summary>
        /// The tempvalue.
        /// </summary>
        private readonly List<string> tempvalue;

        /// <summary>
        /// The item type.
        /// </summary>
        private IFPIO.ObjectEnum ItemType;

        /// <summary>
        /// The indexed reflexive chunk count.
        /// </summary>
        private int indexedReflexiveChunkCount;

        /// <summary>
        /// The indexed reflexive offset.
        /// </summary>
        private int indexedReflexiveOffset;

        /// <summary>
        /// The indexed reflexive translated offset.
        /// </summary>
        private int indexedReflexiveTranslatedOffset;

        /// <summary>
        /// The is nulled out reflexive.
        /// </summary>
        private bool isNulledOutReflexive = true;

        /// <summary>
        /// The selection box value string.
        /// </summary>
        private string selectionBoxValueString;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Indices"/> class.
        /// </summary>
        /// <param name="iEntName">Name of the i ent.</param>
        /// <param name="map">The map.</param>
        /// <param name="iOffsetInChunk">The i offset in chunk.</param>
        /// <param name="iValueType">Type of the i value.</param>
        /// <param name="iIndex">Index of the i.</param>
        /// <remarks></remarks>
        public Indices(string iEntName, Map map, int iOffsetInChunk, IFPIO.ObjectEnum iValueType, IFPIO.Index iIndex)
        {
            this.LineNumber = iIndex.lineNumber;
            this.Dock = DockStyle.Top;
            this.mapIndex = map;
            this.chunkOffset = iOffsetInChunk;
            this.EntName = iEntName;
            this.EntIndex = iIndex;
            this.tempvalue = new List<string>(0);

            

            switch (this.EntIndex.ItemType.ToLower())
            {
                case "ident":
                    {
                        this.ItemType = IFPIO.ObjectEnum.Ident; // Entity.Forms.Meta_Editor.DataValues.ENTType.ident;
                        break;
                    }

                case "stringid":
                    {
                        this.ItemType = IFPIO.ObjectEnum.StringID; // Entity.Forms.Meta_Editor.DataValues.ENTType.sid;
                        break;
                    }

                case "float":
                    {
                        this.ItemType = IFPIO.ObjectEnum.Float; // Entity.Forms.Meta_Editor.DataValues.ENTType.Float;
                        break;
                    }

                case "short":
                    {
                        this.ItemType = IFPIO.ObjectEnum.Short; // Entity.Forms.Meta_Editor.DataValues.ENTType.Int16;
                        break;
                    }

                case "ushort":
                    {
                        this.ItemType = IFPIO.ObjectEnum.UShort; // Entity.Forms.Meta_Editor.DataValues.ENTType.Uint16;
                        break;
                    }

                case "int":
                    {
                        this.ItemType = IFPIO.ObjectEnum.Int; // Entity.Forms.Meta_Editor.DataValues.ENTType.Int32;
                        break;
                    }

                case "uint":
                    {
                        this.ItemType = IFPIO.ObjectEnum.UInt; // Entity.Forms.Meta_Editor.DataValues.ENTType.UInt32;
                        break;
                    }

                case "string32":
                    {
                        this.ItemType = IFPIO.ObjectEnum.String32;

                        // Entity.Forms.Meta_Editor.DataValues.ENTType.string32;
                        break;
                    }

                case "unicodestring64":
                    {
                        this.ItemType = IFPIO.ObjectEnum.UnicodeString64;

                        // Entity.Forms.Meta_Editor.DataValues.ENTType.unicodestring64;
                        break;
                    }

                case "string256":
                    {
                        this.ItemType = IFPIO.ObjectEnum.String256;

                        // Entity.Forms.Meta_Editor.DataValues.ENTType.string256;
                        break;
                    }

                case "unicodestring256":
                    {
                        goto case "string256";
                    }

                default:
                    {
                        this.ItemType = IFPIO.ObjectEnum.Unknown;

                        // Entity.Forms.Meta_Editor.DataValues.ENTType.nothing;
                        break;
                    }
            }

            

            InitializeComponent();
            switch (iValueType)
            {
                case IFPIO.ObjectEnum.Short:
                    {
                        this.label1.Text = "Int16 Block Index";
                        this.ValueType = IFPIO.ObjectEnum.Short;
                        break;
                    }

                case IFPIO.ObjectEnum.Int:
                    {
                        this.label1.Text = "Int32 Block Index";
                        this.ValueType = IFPIO.ObjectEnum.Int;
                        break;
                    }

                case IFPIO.ObjectEnum.UShort:
                    {
                        this.label1.Text = "Uint16 Block Index";
                        this.ValueType = IFPIO.ObjectEnum.UShort;
                        break;
                    }

                case IFPIO.ObjectEnum.UInt:
                    {
                        this.label1.Text = "Uint32 Block Index";
                        this.ValueType = IFPIO.ObjectEnum.UInt;
                        break;
                    }

                case IFPIO.ObjectEnum.Byte:
                    {
                        this.label1.Text = "Byte Block Index";
                        this.ValueType = IFPIO.ObjectEnum.Byte;
                        break;
                    }
            }

            this.label2.Text = this.EntName;
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// The base field_ leave.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        public override void BaseField_Leave(object sender, EventArgs e)
        {
            BinaryWriter BW = new BinaryWriter(mapIndex.SelectedMeta.MS);
            if (mapIndex.Functions.ForMeta.FindMetaByOffset(this.offsetInMap) ==
                mapIndex.SelectedMeta.TagIndex)
            {
                BW.BaseStream.Position = this.offsetInMap - mapIndex.SelectedMeta.offset;
            }
            else
            {
                // ((WinMetaEditor)this.TopLevelControl).ExternalData.Items.Add();
            }

            string tempstring1 = this.comboBox1.Text;
            if (tempstring1.Contains(" Is Invalid. On Line ") || tempstring1.Contains("Something is wrong with this ") ||
                tempstring1.Contains(" : Value is Too Small To Be An Index") ||
                tempstring1.Contains(" : Value is Too Large To Be The Indexer"))
            {
                return;
            }

            if (tempstring1 == "nulled")
            {
                this.Value = -1;
            }

            if (tempstring1.Contains(" : "))
            {
                int counter;
                for (counter = 0; counter < tempstring1.Length; counter++)
                {
                    if (tempstring1[counter] == ' ')
                    {
                        break;
                    }
                }

                this.Value = Convert.ToInt32(tempstring1.Substring(0, counter));
            }

            try
            {
                switch (ValueType)
                {
                    case IFPIO.ObjectEnum.Short:
                        {
                            BW.Write(Convert.ToInt16(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Int:
                        {
                            BW.Write(Convert.ToInt32(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.UShort:
                        {
                            BW.Write(Convert.ToUInt16(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.UInt:
                        {
                            BW.Write(Convert.ToUInt32(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Float:
                        {
                            BW.Write(Convert.ToSingle(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Unknown:
                        {
                            BW.Write(Convert.ToSingle(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Byte:
                        {
                            BW.Write(Convert.ToByte(this.Value));
                            break;
                        }
                }
            }
            catch (Exception ex)
            {
                Global.ShowErrorMsg(
                    "Something is wrong with this " + this.ValueType + this.EntName + " Offset " + this.chunkOffset, ex);
            }
        }

        /// <summary>
        /// The poke.
        /// </summary>
        /// <remarks></remarks>
        public void Poke()
        {
            if (this.isNulledOutReflexive)
            {
                return;
            }

            string tempstring1 = this.comboBox1.Text;
            if (tempstring1.Contains(" Is Invalid. On Line ") || tempstring1.Contains("Something is wrong with this ") ||
                tempstring1.Contains(" : Value is Too Small To Be An Index") ||
                tempstring1.Contains(" : Value is Too Large To Be The Indexer"))
            {
                return;
            }

            if (tempstring1 == "nulled")
            {
                this.Value = -1;
            }

            if (tempstring1.Contains(" : "))
            {
                int counter;
                for (counter = 0; counter < tempstring1.Length; counter++)
                {
                    if (tempstring1[counter] == ' ')
                    {
                        break;
                    }
                }

                this.Value = Convert.ToInt32(tempstring1.Substring(0, counter));
            }

            uint Address = (uint)(this.offsetInMap + mapIndex.SelectedMeta.magic);
            switch (ValueType)
            {
                case IFPIO.ObjectEnum.Short:
                    {
                        RTH_Imports.Poke(Address, (uint)Convert.ToInt16(this.Value), 16);
                        break;
                    }

                case IFPIO.ObjectEnum.Int:
                    {
                        RTH_Imports.Poke(Address, (uint)Convert.ToInt32(this.Value), 32);
                        break;
                    }

                case IFPIO.ObjectEnum.UShort:
                    {
                        RTH_Imports.Poke(Address, Convert.ToUInt16(this.Value), 16);
                        break;
                    }

                case IFPIO.ObjectEnum.UInt:
                    {
                        RTH_Imports.Poke(Address, Convert.ToUInt32(this.Value), 32);
                        break;
                    }

                case IFPIO.ObjectEnum.Float:
                    {
                        RTH_Imports.Poke(Address, RTH_Imports.ConvertFloat(Convert.ToSingle(this.Value)), 32);
                        break;
                    }

                case IFPIO.ObjectEnum.Unknown:
                    {
                        RTH_Imports.Poke(Address, RTH_Imports.ConvertFloat(Convert.ToSingle(this.Value)), 32);
                        break;
                    }

                case IFPIO.ObjectEnum.Byte:
                    {
                        RTH_Imports.Poke(Address, Convert.ToByte(this.Value), 8);
                        break;
                    }
            }
        }

        /// <summary>
        /// The populate.
        /// </summary>
        /// <param name="iOffset">The i offset.</param>
        /// <param name="iIndexedReflexiveOffset">The i indexed reflexive offset.</param>
        /// <remarks></remarks>
        public void Populate(int iOffset, int iIndexedReflexiveOffset)
        {
            this.isNulledOutReflexive = false;
            BinaryReader BR = new BinaryReader(mapIndex.SelectedMeta.MS);

            if (this.EntIndex.reflexiveLayer.ToLower() == "root")
            {
                this.indexedReflexiveOffset = mapIndex.SelectedMeta.offset + this.EntIndex.ReflexiveOffset;
            }
            else if (this.EntIndex.reflexiveLayer.ToLower() == "oneup")
            {
                this.indexedReflexiveOffset = iIndexedReflexiveOffset + this.EntIndex.ReflexiveOffset;
            }

            /*
            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }
            mapIndex.BR.BaseStream.Position = iOffset + this.chunkOffset;
            */
            BR.BaseStream.Position = iOffset + this.chunkOffset;
            this.offsetInMap = mapIndex.SelectedMeta.offset + iOffset + this.chunkOffset;

            switch (ValueType)
            {
                case IFPIO.ObjectEnum.Short:
                    {
                        this.Value = BR.ReadInt16();
                        break;
                    }

                case IFPIO.ObjectEnum.Int:
                    {
                        this.Value = BR.ReadInt32();
                        break;
                    }

                case IFPIO.ObjectEnum.UShort:
                    {
                        this.Value = BR.ReadUInt16();
                        break;
                    }

                case IFPIO.ObjectEnum.UInt:
                    {
                        this.Value = (int)BR.ReadUInt32();
                        break;
                    }

                case IFPIO.ObjectEnum.Byte:
                    {
                        this.Value = BR.ReadByte();
                        break;
                    }
            }

            UpdateSelectionList(false);

            /*
            if (openedMap == true)
                mapIndex.CloseMap();
            */
        }

        /// <summary>
        /// The save.
        /// </summary>
        /// <exception cref="NotImplementedException">
        ///   </exception>
        /// <remarks></remarks>
        public override void Save()
        {
            if (this.isNulledOutReflexive)
            {
                return;
            }

            string tempstring1 = this.comboBox1.Text;
            if (tempstring1.Contains(" Is Invalid. On Line ") || tempstring1.Contains("Something is wrong with this ") ||
                tempstring1.Contains(" : Value is Too Small To Be An Index") ||
                tempstring1.Contains(" : Value is Too Large To Be The Indexer"))
            {
                return;
            }

            if (tempstring1 == "nulled")
            {
                this.Value = -1;
            }

            if (tempstring1.Contains(" : "))
            {
                int counter;
                for (counter = 0; counter < tempstring1.Length; counter++)
                {
                    if (tempstring1[counter] == ' ')
                    {
                        break;
                    }
                }

                this.Value = Convert.ToInt32(tempstring1.Substring(0, counter));
            }

            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }

            try
            {
                mapIndex.BW.BaseStream.Position = this.offsetInMap;
                switch (ValueType)
                {
                    case IFPIO.ObjectEnum.Short:
                        {
                            mapIndex.BW.Write(Convert.ToInt16(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Int:
                        {
                            mapIndex.BW.Write(Convert.ToInt32(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.UShort:
                        {
                            mapIndex.BW.Write(Convert.ToUInt16(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.UInt:
                        {
                            mapIndex.BW.Write(Convert.ToUInt32(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Float:
                        {
                            mapIndex.BW.Write(Convert.ToSingle(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Unknown:
                        {
                            mapIndex.BW.Write(Convert.ToSingle(this.Value));
                            break;
                        }

                    case IFPIO.ObjectEnum.Byte:
                        {
                            mapIndex.BW.Write(Convert.ToByte(this.Value));
                            break;
                        }
                }
            }
            catch //(Exception e)
            {
                MessageBox.Show(
                    "Something is wrong with this " + this.ValueType + this.EntName + " Offset " + this.chunkOffset);
            }

            if (openedMap)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The set focus.
        /// </summary>
        /// <param name="LineToCheck">The line to check.</param>
        /// <remarks></remarks>
        public void SetFocus(int LineToCheck)
        {
            if (this.LineNumber == LineToCheck)
            {
                this.Focus();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// The get items.
        /// </summary>
        /// <param name="readAll">The read all.</param>
        /// <remarks></remarks>
        private void GetItems(bool readAll)
        {
            this.tempvalue.Clear();
            if (this.indexedReflexiveChunkCount < 0)
            {
                tempvalue.Add("Index " + this.EntName + " Is Invalid. On Line " + this.EntIndex.lineNumber);
                return;
            }

            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }

            int countervalue = 0;
            int maxcount = this.indexedReflexiveChunkCount;
            if (readAll == false && this.Value > -1)
            {
                countervalue = this.Value;
                maxcount = this.Value + 1;
            }
            else if (readAll == false)
            {
                countervalue = 0;
                maxcount = 0;
            }

            string tempValueString = string.Empty;
            for (int counter = countervalue; counter < maxcount; counter++)
            {
                try
                {
                    mapIndex.BR.BaseStream.Position = this.indexedReflexiveTranslatedOffset +
                                                      (this.EntIndex.ReflexiveSize * counter) + this.EntIndex.ItemOffset;
                    switch (this.EntIndex.ItemType.ToLower())
                    {
                        case "short":
                            {
                                tempValueString = counter + " : " + mapIndex.BR.ReadInt16();
                                break;
                            }

                        case "int":
                            {
                                tempValueString = counter + " : " + mapIndex.BR.ReadInt32();
                                break;
                            }

                        case "ushort":
                            {
                                tempValueString = counter + " : " + mapIndex.BR.ReadUInt16();
                                break;
                            }

                        case "uint":
                            {
                                tempValueString = counter + " : " + mapIndex.BR.ReadUInt32();
                                break;
                            }

                        case "float":
                            {
                                tempValueString = counter + " : " + mapIndex.BR.ReadSingle();
                                break;
                            }

                        case "stringid":
                            {
                                int sidIndexer = mapIndex.BR.ReadInt16();
                                if (sidIndexer > -1 && sidIndexer < mapIndex.Strings.Name.Length)
                                {
                                    tempValueString = counter + " : " + mapIndex.Strings.Name[sidIndexer];
                                }
                                else
                                {
                                    tempValueString = counter + " : Not a String ID";
                                }

                                break;
                            }

                        case "ident":
                            {
                                int tempint = mapIndex.BR.ReadInt32();
                                int tagIndex = mapIndex.Functions.ForMeta.FindMetaByID(tempint);
                                if (tagIndex != -1)
                                {
                                    if (tempint != -1)
                                    {
                                        tempValueString = counter + " : " + mapIndex.MetaInfo.TagType[tagIndex] + " - " +
                                                          mapIndex.FileNames.Name[tagIndex];
                                    }
                                    else
                                    {
                                        tempValueString = counter +
                                                          " : Probably is NOT an ident, choose another data type";
                                    }
                                }
                                else if (tagIndex == -1)
                                {
                                    tempValueString = counter + " : nulled ident";
                                }
                                else
                                {
                                    tempValueString = counter + " : Probably is NOT an ident, choose another data type";
                                }

                                break;
                            }

                        case "string32":
                            {
                                Encoding decode = Encoding.UTF8;
                                byte[] tempbytes = mapIndex.BR.ReadBytes(32);
                                tempValueString = counter + " : " + decode.GetString(tempbytes);
                                break;
                            }

                        case "unicodestring64":
                            {
                                Encoding decode = Encoding.UTF8;
                                byte[] tempbytes = mapIndex.BR.ReadBytes(64);
                                tempValueString = counter + " : " + decode.GetString(tempbytes);
                                break;
                            }

                        case "string256":
                            {
                                Encoding decode = Encoding.UTF8;
                                byte[] tempbytes = mapIndex.BR.ReadBytes(256);
                                tempValueString = counter + " : " + decode.GetString(tempbytes);
                                break;
                            }

                        case "unicodestring256":
                            {
                                goto case "string256";
                            }
                    }
                }
                catch
                {
                    tempValueString = counter + " : Error with index on line number " + this.EntIndex.lineNumber;
                }

                tempvalue.Add(tempValueString);
                if (counter == this.Value)
                {
                    this.selectionBoxValueString = tempValueString;
                }
            }

            tempvalue.Add("nulled");
            if (openedMap)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The read reflexive.
        /// </summary>
        /// <remarks></remarks>
        private void ReadReflexive()
        {
            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }

            mapIndex.BR.BaseStream.Position = this.indexedReflexiveOffset;
            this.indexedReflexiveChunkCount = mapIndex.BR.ReadInt32();
            this.indexedReflexiveTranslatedOffset = mapIndex.BR.ReadInt32() - mapIndex.SelectedMeta.magic;
            if (openedMap)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The update selection list.
        /// </summary>
        /// <param name="readAll">The read all.</param>
        /// <remarks></remarks>
        private void UpdateSelectionList(bool readAll)
        {
            this.ReadReflexive();
            this.GetItems(readAll);
            this.comboBox1.Items.Clear();
            this.comboBox1.Items.AddRange(tempvalue.ToArray());

            if (this.Value >= this.indexedReflexiveChunkCount)
            {
                this.comboBox1.Items.Add(this.Value + " : Value is Too Large To Be The Indexer");
                this.comboBox1.SelectedIndex = this.comboBox1.Items.Count - 1;
            }
            else if (this.Value < -1)
            {
                this.comboBox1.Items.Add(this.Value + " : Value is Too Small To Be An Index");
                this.comboBox1.SelectedIndex = this.comboBox1.Items.Count - 1;
            }
            else if (this.Value == -1)
            {
                if (readAll == false)
                {
                    this.comboBox1.Text = "nulled";
                }
                else
                {
                    this.comboBox1.SelectedIndex = this.indexedReflexiveChunkCount;
                }
            }
            else
            {
                if (readAll == false)
                {
                    this.comboBox1.Text = this.selectionBoxValueString;
                }
                else
                {
                    this.comboBox1.SelectedIndex = this.Value;
                }
            }
        }

        /// <summary>
        /// The combo box 1_ drop down.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void comboBox1_DropDown(object sender, EventArgs e)
        {
            // if (tempvalue.Count != ((ComboBox)sender).Items.Count)
            UpdateSelectionList(true);
            this.comboBox1.SelectedIndex = this.Value;
        }

        #endregion
    }
}