﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="MetaEditor.cs" company="">
//   
// </copyright>
// <summary>
//   The win meta editor.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.MetaEditor2
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.IO;
    using System.Text;
    using System.Windows.Forms;

    using entity.MapForms;

    using HaloMap.Map;
    using HaloMap.Meta;
    using HaloMap.Plugins;

    using MetaEditor;

    /// <summary>
    /// The win meta editor.
    /// </summary>
    /// <remarks></remarks>
    public partial class WinMetaEditor : Form
    {
        #region Constants and Fields

        /// <summary>
        /// The map.
        /// </summary>
        private readonly Map map;

        /// <summary>
        /// The meta.
        /// </summary>
        private readonly Meta meta;

        /// <summary>
        /// The ms backup.
        /// </summary>
        private readonly MemoryStream msBackup;

        /// <summary>
        /// The ref data.
        /// </summary>
        private readonly List<reflexiveData> refData = new List<reflexiveData>();

        /// <summary>
        /// The br.
        /// </summary>
        private BinaryReader BR;

        /// <summary>
        /// The bw.
        /// </summary>
        private BinaryWriter BW;

        /// <summary>
        /// The ifp.
        /// </summary>
        private IFPIO ifp;

        /// <summary>
        /// The old meta.
        /// </summary>
        private Meta oldMeta;

        /// <summary>
        /// The tv hidden nodes.
        /// </summary>
        private TreeView tvHiddenNodes = new TreeView();

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="WinMetaEditor"/> class.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="tagNumber">The tag number.</param>
        /// <param name="map">The map.</param>
        /// <remarks></remarks>
        public WinMetaEditor(MapForm sender, int tagNumber, Map map)
        {
            InitializeComponent();

            this.map = map;

            Meta temp = map.SelectedMeta;
            sender.LoadMeta(tagNumber);
            this.Owner = sender;
            meta = map.SelectedMeta;
            map.SelectedMeta = temp;

            this.Text = map.filePath.Substring(map.filePath.LastIndexOf('\\') + 1).ToUpper() + ": [" +
                        meta.type.ToLower() + "] " + meta.name;

            BR = new BinaryReader(meta.MS);
            BW = new BinaryWriter(meta.MS);

            // Create a backup of the Tags memory stream, for restoring, comparing, etc
            msBackup = new MemoryStream(meta.MS.ToArray());

            createTreeListing();
            this.treeViewTagReflexives.Sort();
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// The reload meta for same reflexive.
        /// </summary>
        /// <param name="offset">The offset.</param>
        /// <exception cref="Exception">
        ///   </exception>
        /// <remarks></remarks>
        public void ReloadMetaForSameReflexive(int offset)
        {
            reflexiveData rd = (reflexiveData)treeViewTagReflexives.SelectedNode.Tag;
            map.OpenMap(MapTypes.Internal);

            for (int counter = 0; counter < this.panelMetaEditor.Controls.Count; counter++)
            {
                switch (this.panelMetaEditor.Controls[counter].GetType().ToString())
                {
                    case "Entity.Forms.MetaEditor.SID":
                        {
                            ((SID)this.panelMetaEditor.Controls[counter]).Populate(
                                offset, rd.inTagNumber == meta.TagIndex);
                            break;
                        }

                        /*
                    case "Entity.Forms.MetaEditor.ReflexiveControl":
                        {
                            if (reloadreflexive == false)
                                break;
                            ((ReflexiveControl)this.panelMetaEditor.Controls[counter]).LoadMetaIntoControls(offset, offset);
                            break;
                        }
                    */
                    case "Entity.Forms.MetaEditor.Ident":
                        {
                            ((Ident)this.panelMetaEditor.Controls[counter]).Populate(
                                offset, rd.inTagNumber == meta.TagIndex);
                            break;
                        }

                    case "Entity.Forms.MetaEditor.EntStrings":
                        {
                            ((EntStrings)this.panelMetaEditor.Controls[counter]).Populate(
                                offset, rd.inTagNumber == meta.TagIndex);
                            break;
                        }

                    case "Entity.Forms.MetaEditor.Bitmask":
                        {
                            ((Bitmask)this.panelMetaEditor.Controls[counter]).Populate(
                                offset, rd.inTagNumber == meta.TagIndex);
                            break;
                        }

                    case "Entity.Forms.MetaEditor.DataValues":
                        {
                            ((DataValues)this.panelMetaEditor.Controls[counter]).Populate(
                                offset, rd.inTagNumber == meta.TagIndex);
                            break;
                        }

                    case "Entity.Forms.MetaEditor.Enums":
                        {
                            ((Enums)this.panelMetaEditor.Controls[counter]).Populate(
                                offset, rd.inTagNumber == meta.TagIndex);
                            break;
                        }

                    case "Entity.Forms.MetaEditor.Indices":
                        {
                            ((Indices)this.panelMetaEditor.Controls[counter]).Populate(
                                offset, ((reflexiveData)rd.node.Parent.Tag).baseOffset);
                            break;
                        }

                    default:
                        throw new Exception("Unhandled type: " + this.panelMetaEditor.Controls[counter].GetType());
                }

                int temp = ((BaseField)this.panelMetaEditor.Controls[counter]).offsetInMap;

                // Take into account that idents actually start at -4 due to tags preceding
                if (this.panelMetaEditor.Controls[counter].GetType().ToString() == "Entity.Forms.MetaEditor.Ident")
                {
                    temp -= 4;
                }

                int tagIndex = map.Functions.ForMeta.FindMetaByOffset(temp);

                // ****************
                // base offset = location of reflexive information (count & tag 0 offset)
                // tag offset is location of actual data!
                // ****************
                try
                {
                    if (tagIndex == meta.TagIndex)
                    {
                        toolTip1.SetToolTip(
                            panelMetaEditor.Controls[counter].Controls[0], 
                            "offset in reflexive: " + ((BaseField)this.panelMetaEditor.Controls[counter]).chunkOffset +
                            "\noffset in tag: " + rd.reflexive.offset + "\noffset in tag: " + (temp - meta.offset) +
                            "\noffset in map: " + temp);
                    }
                    else
                    {
                        toolTip1.SetToolTip(
                            panelMetaEditor.Controls[counter].Controls[0], 
                            "offset in reflexive: " + ((BaseField)this.panelMetaEditor.Controls[counter]).chunkOffset +
                            "\noffset in tag: " + (temp - map.MetaInfo.Offset[tagIndex]) + "\noffset in map: " + temp);
                    }
                }
                catch
                {
                }
            }

            map.CloseMap();
        }

        /// <summary>
        /// The check selection in current tag.
        /// </summary>
        /// <returns>The check selection in current tag.</returns>
        /// <remarks></remarks>
        public bool checkSelectionInCurrentTag()
        {
            return meta.TagIndex == ((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).inTagNumber;
        }

        /// <summary>
        /// The goto offset.
        /// </summary>
        /// <param name="offset">The offset.</param>
        /// <remarks></remarks>
        public void gotoOffset(int offset)
        {
            foreach (Meta.Item mi in this.meta.items)
            {
                if (!(mi is Meta.Reflexive))
                {
                    continue;
                }

                map.OpenMap(MapTypes.Internal);
                map.BR.BaseStream.Position = mi.mapOffset;
                int count = map.BR.ReadInt32();
                int ofs = map.BR.ReadInt32() - meta.magic;
                map.CloseMap();

                for (int i = 0; i < count; i++)
                {
                    if ((ofs + ((Meta.Reflexive)mi).chunksize * i) == offset)
                    {
                        treeViewTagReflexives.SelectedNode = findNodeOffset(treeViewTagReflexives.Nodes[0].Nodes, ofs);
                        return;
                    }
                }
            }
        }

        /// <summary>
        /// The load tree reflexives.
        /// </summary>
        /// <param name="metaOffset">The meta offset.</param>
        /// <param name="items">The items.</param>
        /// <param name="enabled">The enabled.</param>
        /// <returns></returns>
        /// <remarks></remarks>
        public TreeNode[] loadTreeReflexives(int metaOffset, object[] items, bool enabled)
        {
            List<TreeNode> tns = new List<TreeNode>();
            foreach (object o in items)
            {
                if (o is IFPIO.Reflexive)
                {
                    IFPIO.Reflexive IFPR = (IFPIO.Reflexive)o;
                    TreeNode tn = new TreeNode(IFPR.name);
                    reflexiveData rd = new reflexiveData();
                    tn.Tag = rd;
                    rd.node = tn;
                    rd.reflexive = IFPR;
                    tn.ForeColor = Color.LightGray;
                    tn.Name = IFPR.name;

                    if (enabled)
                    {
                        map.BR.BaseStream.Position = IFPR.offset + metaOffset;
                        rd.chunkCount = map.BR.ReadInt32();
                        if (rd.chunkCount > 0)
                        {
                            rd.chunkSelected = 0;
                            tn.ForeColor = Color.Black;
                            rd.baseOffset = map.BR.ReadInt32() - meta.magic;

                            // Some listings are actually in other tags!
                            // Check [BLOC] "objects\\vehicles\\wraith\\garbage\\wing_boost\\wing_boost"
                            // > attachments[0] = [BLOC] "objects\\vehicles\\ghost\\garbage\\seat\\seat"
                            rd.inTagNumber = map.Functions.ForMeta.FindMetaByOffset(rd.baseOffset);
                            if (rd.inTagNumber == meta.TagIndex)
                            {
                                rd.baseOffset -= meta.offset;
                            }
                            else
                            {
                                tn.ForeColor = Color.Red;
                                tn.ToolTipText = "Data Source Located in:\n[" +
                                                 map.MetaInfo.TagType[rd.inTagNumber].ToLower() + "] " +
                                                 map.FileNames.Name[rd.inTagNumber].ToLower();
                            }
                        }
                    }

                    tn.Text += " [" + rd.chunkCount + "]";

                    refData.Add(rd);

                    // Add if non-existant, otherwise update Text
                    if (rd.inTagNumber == meta.TagIndex)
                    {
                        tn.Nodes.AddRange(
                            loadTreeReflexives(
                                meta.offset + rd.baseOffset + rd.chunkSelected * rd.reflexive.chunkSize, 
                                IFPR.items, 
                                rd.chunkCount != 0));
                    }
                    else
                    {
                        tn.Nodes.AddRange(
                            loadTreeReflexives(
                                rd.baseOffset + rd.chunkSelected * rd.reflexive.chunkSize, 
                                IFPR.items, 
                                rd.chunkCount != 0));
                    }

                    if (rd.chunkCount != 0 | !cbHideUnused.Checked)
                    {
                        tns.Add(tn);
                    }
                }
            }

            return tns.ToArray();
        }

        /// <summary>
        /// Displays an info pane and keeps it open for an amount of time.
        /// </summary>
        /// <param name="info">The text to display in the Info Pane</param>
        /// <param name="time">The time to display the info pane, in milliseconds</param>
        /// <remarks></remarks>
        public void showInfoBox(string info, int time)
        {
            panelInfoPane.Text = info;

            panelInfoPane.Visible = true;
            timer1.Stop();
            if (time > 0)
            {
                timer1.Interval = time;
                timer1.Start();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// This is used to update the reflexive label
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        /// <remarks></remarks>
        private void WinMEControl_LostFocus(object sender, EventArgs e)
        {
            ToolStripComboBox tscb =
                (ToolStripComboBox)this.toolStripNavigation.Items[toolStripNavigation.Items.Count - 1];
            tscb.Enabled = false;
            tscb.Items[tscb.SelectedIndex] = tscb.SelectedIndex + " : \"" + ((Control)sender).Text + "\"";
            tscb.Enabled = true;
        }

        /// <summary>
        /// The win meta editor_ activated.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void WinMetaEditor_Activated(object sender, EventArgs e)
        {
            oldMeta = map.SelectedMeta;
            map.SelectedMeta = meta;
            if (((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).chunkCount > 0)
            {
                ReloadMetaForSameReflexive(((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).baseOffset);
            }
        }

        /// <summary>
        /// The win meta editor_ deactivate.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void WinMetaEditor_Deactivate(object sender, EventArgs e)
        {
            if (oldMeta != null)
            {
                map.SelectedMeta = oldMeta;
            }
        }

        /// <summary>
        /// The win meta editor_ form closing.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void WinMetaEditor_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Focus the Save button to make sure that the current field is updated by BaseField_Leave() function
            this.btnSave.Focus();

            byte[] bn = meta.MS.ToArray();
            byte[] bo = msBackup.ToArray();
            List<int> changedOffsets = new List<int>();
            for (int i = 0; i < bn.Length; i++)
            {
                if (bn[i] != bo[i])
                {
                    changedOffsets.Add(i);
                }
            }

            if (changedOffsets.Count > 0)
            {
                DialogResult dr = MessageBox.Show(
                    "Changes were made, do you wish to save?", "Save Changes?", MessageBoxButtons.YesNoCancel);
                if (dr == DialogResult.Yes)
                {
                    btnSave_Click(this, null);
                }
                else if (dr == DialogResult.Cancel)
                {
                    e.Cancel = true;
                }
            }
        }

        /// <summary>
        /// The add reflexive.
        /// </summary>
        /// <param name="ts">The ts.</param>
        /// <param name="text">The text.</param>
        /// <remarks></remarks>
        private void addReflexive(ToolStrip ts, string text)
        {
            ToolStripComboBox tsbc = new ToolStripComboBox();
            tsbc.AutoSize = false;
            tsbc.DropDownStyle = ComboBoxStyle.DropDownList;
            tsbc.DropDown += tsbc_DropDown;
            tsbc.DropDownClosed += tsbc_DropDownClosed;
            tsbc.SelectedIndexChanged += tsbc_SelectedIndexChanged;
            tsbc.Size = new Size(49, 21);
            ts.Items.Insert(0, tsbc);
            if (text.Length > 23)
            {
                string t = text.Substring(0, 10) + "..." + text.Substring(text.Length - 10);
                text = t;
            }

            ToolStripLabel tsl = new ToolStripLabel(text);
            ts.Items.Insert(0, tsl);
            ToolStripSeparator tss = new ToolStripSeparator();
            ts.Items.Insert(0, tss);
        }

        /// <summary>
        /// The btn save_ click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void btnSave_Click(object sender, EventArgs e)
        {
            map.OpenMap(MapTypes.Internal);
            map.BW.BaseStream.Position = meta.offset;
            map.BW.Write(meta.MS.ToArray());
            map.CloseMap();
            showInfoBox("Tag Saved.", 1000);
        }

        /// <summary>
        /// The cb hide unused_ checked changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void cbHideUnused_CheckedChanged(object sender, EventArgs e)
        {
            createTreeListing();
        }

        /// <summary>
        /// The create tree listing.
        /// </summary>
        /// <remarks></remarks>
        private void createTreeListing()
        {
            ifp = IFPHashMap.GetIfp(meta.type, map);

            treeViewTagReflexives.Nodes.Clear();
            treeViewTagReflexives.Nodes.Add("0", ".:[ MAIN ]:.");
            reflexiveData rd = new reflexiveData();
            treeViewTagReflexives.Nodes[0].Tag = rd;
            rd.node = treeViewTagReflexives.Nodes[0];
            rd.chunkCount = 1;
            rd.chunkSelected = 0;
            rd.baseOffset = 0; // meta.offset;
            rd.inTagNumber = meta.TagIndex;
            refData.Add(rd);

            map.OpenMap(MapTypes.Internal);

            treeViewTagReflexives.Nodes[0].Nodes.AddRange(loadTreeReflexives(meta.offset, ifp.items, true));

            map.CloseMap();

            // treeViewTagReflexives.ExpandAll();
            treeViewTagReflexives.Nodes[0].Expand();
        }

        /// <summary>
        /// The find node offset.
        /// </summary>
        /// <param name="tns">The tns.</param>
        /// <param name="offset">The offset.</param>
        /// <returns></returns>
        /// <remarks></remarks>
        private TreeNode findNodeOffset(TreeNodeCollection tns, int offset)
        {
            TreeNode tn = null;
            for (int i = 0; i < tns.Count; i++)
            {
                int ofs = ((reflexiveData)tns[i].Tag).baseOffset;
                if (((reflexiveData)tns[i].Tag).inTagNumber == meta.TagIndex)
                {
                    ofs += meta.offset;
                }

                if (ofs == offset)
                {
                    return tns[i];
                }

                if (tns[i].Nodes.Count > 0)
                {
                    tn = findNodeOffset(tns[i].Nodes, offset);
                }

                if (tn != null)
                {
                    break;
                }
            }

            return tn;
        }

        /// <summary>
        /// The load controls.
        /// </summary>
        /// <param name="Location">The location.</param>
        /// <remarks></remarks>
        private void loadControls(TreeNode Location)
        {
            reflexiveData rd = (reflexiveData)Location.Tag;
            int chNum = rd.chunkSelected;
            bool enabled = rd.chunkCount != 0;
            int metaOffset = rd.baseOffset;

            Panel panel = this.panelMetaEditor;
            panel.Visible = false;

            // Dispose of each control and also add a safety  in case we somehow go into an endless loop (max 1000)
            int safetyCount = 1000;
            while (panel.Controls.Count > 0 && safetyCount > 0)
            {
                panel.Controls[0].Dispose();
                safetyCount--;
            }

            // Should be empty, but clear anyways.
            panel.Controls.Clear();

            object[] o;
            if (rd.reflexive == null)
            {
                o = ifp.items;
            }
            else
            {
                o = rd.reflexive.items;
            }

            //bool labelFound = false;
            int tabIndex = 0;
            foreach (IFPIO.BaseObject ctl in o)
            {
                tabIndex++;
                switch (ctl.ObjectType)
                {
                    case IFPIO.ObjectEnum.Struct:
                        {
                            /*
                            if (Meta_Editor.MetaEditor.ShowReflexives == false)
                                break;
                            // tempLabel is a blank space located above reflexives
                            Label tempLabel = new Label();
                            tempLabel.AutoSize = true;
                            tempLabel.Location = new System.Drawing.Point(0, 0);
                            tempLabel.Name = "label1";
                            tempLabel.Dock = DockStyle.Top;
                            tempLabel.Size = new System.Drawing.Size(35, 13);
                            tempLabel.TabIndex = tabIndex;

                            // tempReflexive is the reflexive and all data (incl other reflexives) within it
                            ReflexiveControl tempReflexive = new ReflexiveControl(map, meta.offset, ((IFPIO.Reflexive)ctl).HasCount, ctl.lineNumber, this);
                            //tempReflexive.Location = new System.Drawing.Point(10, 0);
                            tempReflexive.Name = "reflexive";
                            tempReflexive.TabIndex = tabIndex;
                            tempReflexive.LoadENTControls((IFPIO.Reflexive)ctl, ((IFPIO.Reflexive)ctl).items,
                                                           true, 0, ref tabIndex, ctl.offset.ToString());

                            // Label, Combobox & Button are always added ( = 3)
                            if (!(tempReflexive.Controls.Count <= 2 && skipEmptyReflex))
                            {
                                panelMetaEditor.Controls.Add(tempLabel);
                                panelMetaEditor.Controls[panelMetaEditor.Controls.Count - 1].BringToFront();
                                panelMetaEditor.Controls.Add(tempReflexive);
                                panelMetaEditor.Controls[panelMetaEditor.Controls.Count - 1].BringToFront();
                            }
                            */
                            break;
                        }

                    case IFPIO.ObjectEnum.Ident:
                        {
                            if (MetaEditor.ShowIdents == false)
                            {
                                break;
                            }

                            Ident tempident = new Ident(
                                ctl.name, map, ctl.offset, ((IFPIO.Ident)ctl).hasTagType, ctl.lineNumber);
                            tempident.Name = "ident";
                            tempident.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempident.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            tempident.Tag = "[" + tempident.Controls[2].Text + "] " + tempident.Controls[1].Text;

                            // tempident.Controls[1].ContextMenuStrip = identContext;
                            panelMetaEditor.Controls.Add(tempident);
                            tempident.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.StringID:
                        {
                            if (MetaEditor.ShowSIDs == false)
                            {
                                break;
                            }

                            SID tempSID = new SID(ctl.name, map, ctl.offset, ctl.lineNumber);
                            tempSID.Name = "sid";
                            tempSID.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempSID.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempSID);
                            tempSID.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Float:
                        {
                            if (MetaEditor.ShowFloats == false)
                            {
                                break;
                            }

                            DataValues tempFloat = new DataValues(
                                ctl.name, map, ctl.offset, IFPIO.ObjectEnum.Float, ctl.lineNumber);
                            tempFloat.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempFloat.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempFloat);
                            tempFloat.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.String32:
                        {
                            if (MetaEditor.ShowString32s == false && ctl.ObjectType == IFPIO.ObjectEnum.String32)
                            {
                                break;
                            }

                            EntStrings tempstring = new EntStrings(
                                ctl.name, 
                                map, 
                                ctl.offset, 
                                ((IFPIO.IFPString)ctl).size, 
                                ((IFPIO.IFPString)ctl).type, 
                                ctl.lineNumber);
                            tempstring.Name = "string";
                            tempstring.TabIndex = tabIndex;
                            panelMetaEditor.Controls.Add(tempstring);
                            if (enabled)
                            {
                                tempstring.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            tempstring.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.UnicodeString256:
                        {
                            if (MetaEditor.ShowUnicodeString256s == false)
                            {
                                break;
                            }

                            goto case IFPIO.ObjectEnum.String32;
                        }

                    case IFPIO.ObjectEnum.String256:
                        {
                            if (MetaEditor.ShowString256s == false)
                            {
                                break;
                            }

                            goto case IFPIO.ObjectEnum.String32;
                        }

                    case IFPIO.ObjectEnum.UnicodeString64:
                        {
                            if (MetaEditor.ShowUnicodeString64s == false)
                            {
                                break;
                            }

                            goto case IFPIO.ObjectEnum.String32;
                        }

                    case IFPIO.ObjectEnum.String:
                        {
                            if (MetaEditor.ShowString32s == false && ctl.ObjectType == IFPIO.ObjectEnum.String32)
                            {
                                break;
                            }

                            EntStrings tempstring = new EntStrings(
                                ctl.name, 
                                map, 
                                ctl.offset, 
                                ((IFPIO.IFPString)ctl).size, 
                                ((IFPIO.IFPString)ctl).type, 
                                ctl.lineNumber);
                            tempstring.Name = "string";
                            tempstring.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempstring.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempstring);
                            tempstring.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Int:
                        {
                            if (((IFPIO.IFPInt)ctl).entIndex.nulled)
                            {
                                if ((MetaEditor.ShowInts == false && ctl.ObjectType == IFPIO.ObjectEnum.Int) ||
                                    (MetaEditor.ShowShorts == false && ctl.ObjectType == IFPIO.ObjectEnum.Short) ||
                                    (MetaEditor.ShowUshorts == false && ctl.ObjectType == IFPIO.ObjectEnum.UShort) ||
                                    (MetaEditor.ShowUints == false && ctl.ObjectType == IFPIO.ObjectEnum.UInt))
                                {
                                    break;
                                }

                                DataValues tempdatavalues = new DataValues(
                                    ctl.name, map, ctl.offset, ctl.ObjectType, ctl.lineNumber);
                                tempdatavalues.TabIndex = tabIndex;
                                if (enabled)
                                {
                                    tempdatavalues.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                                }

                                panelMetaEditor.Controls.Add(tempdatavalues);
                                tempdatavalues.BringToFront();
                            }
                            else
                            {
                                if ((MetaEditor.ShowBlockIndex32s == false &&
                                     (ctl.ObjectType == IFPIO.ObjectEnum.Int | ctl.ObjectType == IFPIO.ObjectEnum.UInt)) ||
                                    (MetaEditor.ShowBlockIndex16s == false &&
                                     (ctl.ObjectType == IFPIO.ObjectEnum.Short |
                                      ctl.ObjectType == IFPIO.ObjectEnum.UShort)) ||
                                    (MetaEditor.ShowBlockIndex8s == false && ctl.ObjectType == IFPIO.ObjectEnum.Byte))
                                {
                                    break;
                                }

                                Indices tempdatavalues = new Indices(
                                    ctl.name, map, ctl.offset, ctl.ObjectType, ((IFPIO.IFPInt)ctl).entIndex);
                                tempdatavalues.TabIndex = tabIndex;
                                panelMetaEditor.Controls.Add(tempdatavalues);
                                tempdatavalues.BringToFront();
                            }

                            break;
                        }

                    case IFPIO.ObjectEnum.Short:
                        {
                            goto case IFPIO.ObjectEnum.Int;
                        }

                    case IFPIO.ObjectEnum.UShort:
                        {
                            goto case IFPIO.ObjectEnum.Int;
                        }

                    case IFPIO.ObjectEnum.UInt:
                        {
                            goto case IFPIO.ObjectEnum.Int;
                        }

                    case IFPIO.ObjectEnum.Unknown:
                        {
                            if (MetaEditor.ShowUndefineds == false)
                            {
                                break;
                            }

                            DataValues tempUnknown = new DataValues(
                                ctl.name, map, ctl.offset, IFPIO.ObjectEnum.Unknown, ctl.lineNumber);
                            tempUnknown.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempUnknown.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempUnknown);
                            tempUnknown.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Byte_Flags:
                        {
                            if (MetaEditor.ShowBitmask8s == false)
                            {
                                break;
                            }

                            Bitmask tempbitmask = new Bitmask(
                                ctl.name, 
                                map, 
                                ctl.offset, 
                                ((IFPIO.Bitmask)ctl).bitmaskSize, 
                                ((IFPIO.Bitmask)ctl).options, 
                                ctl.lineNumber);
                            tempbitmask.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempbitmask.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempbitmask);
                            tempbitmask.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Word_Flags:
                        {
                            if (MetaEditor.ShowBitmask16s == false)
                            {
                                break;
                            }

                            Bitmask tempbitmask = new Bitmask(
                                ctl.name, 
                                map, 
                                ctl.offset, 
                                ((IFPIO.Bitmask)ctl).bitmaskSize, 
                                ((IFPIO.Bitmask)ctl).options, 
                                ctl.lineNumber);
                            tempbitmask.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempbitmask.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempbitmask);
                            tempbitmask.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Long_Flags:
                        {
                            if (MetaEditor.ShowBitmask32s == false)
                            {
                                break;
                            }

                            Bitmask tempbitmask = new Bitmask(
                                ctl.name, 
                                map, 
                                ctl.offset, 
                                ((IFPIO.Bitmask)ctl).bitmaskSize, 
                                ((IFPIO.Bitmask)ctl).options, 
                                ctl.lineNumber);
                            tempbitmask.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempbitmask.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempbitmask);
                            tempbitmask.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Char_Enum:
                        {
                            if (MetaEditor.ShowEnum8s == false)
                            {
                                break;
                            }

                            Enums tempenum = new Enums(
                                ctl.name, 
                                map, 
                                ctl.offset, 
                                ((IFPIO.IFPEnum)ctl).enumSize, 
                                ((IFPIO.IFPEnum)ctl).options, 
                                ctl.lineNumber);
                            tempenum.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempenum.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempenum);
                            tempenum.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Enum:
                        {
                            if (MetaEditor.ShowEnum16s == false)
                            {
                                break;
                            }

                            Enums tempenum = new Enums(
                                ctl.name, 
                                map, 
                                ctl.offset, 
                                ((IFPIO.IFPEnum)ctl).enumSize, 
                                ((IFPIO.IFPEnum)ctl).options, 
                                ctl.lineNumber);
                            tempenum.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempenum.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempenum);
                            tempenum.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Long_Enum:
                        {
                            if (MetaEditor.ShowEnum32s == false)
                            {
                                break;
                            }

                            Enums tempenum = new Enums(
                                ctl.name, 
                                map, 
                                ctl.offset, 
                                ((IFPIO.IFPEnum)ctl).enumSize, 
                                ((IFPIO.IFPEnum)ctl).options, 
                                ctl.lineNumber);
                            tempenum.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempenum.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            panelMetaEditor.Controls.Add(tempenum);
                            tempenum.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.Byte:
                        {
                            if (((IFPIO.IFPByte)ctl).entIndex.nulled)
                            {
                                if (MetaEditor.ShowBytes == false)
                                {
                                    break;
                                }

                                DataValues tempByte = new DataValues(
                                    ctl.name, map, ctl.offset, IFPIO.ObjectEnum.Byte, ctl.lineNumber);
                                tempByte.TabIndex = tabIndex;
                                panelMetaEditor.Controls.Add(tempByte);
                                tempByte.BringToFront();
                            }
                            else
                            {
                                if (MetaEditor.ShowBlockIndex8s == false)
                                {
                                    break;
                                }

                                Indices tempdatavalues = new Indices(
                                    ctl.name, map, ctl.offset, ctl.ObjectType, ((IFPIO.IFPByte)ctl).entIndex);
                                tempdatavalues.TabIndex = tabIndex;
                                panelMetaEditor.Controls.Add(tempdatavalues);
                                tempdatavalues.BringToFront();
                            }

                            break;
                        }

                    case IFPIO.ObjectEnum.Unused:
                        {
                            DataValues tempUnknown = new DataValues(
                                ctl.name, map, ctl.offset, IFPIO.ObjectEnum.Unused, ctl.lineNumber);
                            tempUnknown.TabIndex = tabIndex;
                            if (enabled)
                            {
                                tempUnknown.Populate(metaOffset, rd.inTagNumber == meta.TagIndex);
                            }

                            if (tempUnknown.ValueType == IFPIO.ObjectEnum.Unused)
                            {
                                tempUnknown.textBox1.Text = "unused (size " + ((IFPIO.Unused)ctl).size + ")";
                            }

                            panelMetaEditor.Controls.Add(tempUnknown);
                            tempUnknown.BringToFront();
                            break;
                        }

                    case IFPIO.ObjectEnum.TagType:
                        continue;
                    default:
                        break;
                }

                if (ctl.ObjectType != IFPIO.ObjectEnum.Struct)
                {
                    int temp = ctl.offset;

                    // Take into account that idents actually start at -4 due to tags preceding
                    if (ctl.ObjectType == IFPIO.ObjectEnum.Ident)
                    {
                        temp -= 4;
                    }

                    toolTip1.SetToolTip(
                        panelMetaEditor.Controls[0].Controls[0], 
                        "offset in reflexive: " + temp + "\noffset in tag: " + (rd.baseOffset + temp) +
                        "\noffset in map: " + (rd.baseOffset + meta.offset + temp));
                }
            }

            panel.Visible = true;
        }

        /// <summary>
        /// Loads all the labels into the dropdown selection box.
        /// </summary>
        /// <param name="rd">The rd.</param>
        /// <returns></returns>
        /// <remarks></remarks>
        private string[] loadLabels(reflexiveData rd)
        {
            List<string> ls = new List<string>();
            IFPIO.BaseObject bo = null;

            if (rd.reflexive.label == string.Empty)
            {
                return new string[rd.chunkCount];
            }

            for (int i = 0; i <= rd.reflexive.items.Length; i++)
            {
                if (rd.reflexive.label == ((IFPIO.BaseObject)rd.reflexive.items[i]).name)
                {
                    bo = (IFPIO.BaseObject)rd.reflexive.items[i];
                    break;
                }
            }

            // If label does not exist, then return an empty array
            if (bo == null)
            {
                return new string[rd.chunkCount];
            }

            BinaryReader BR = new BinaryReader(meta.MS);

            // baseOffset contains the offset to the 0 index of the reflexive
            int baseOffset = rd.baseOffset - (rd.reflexive.chunkSize * rd.chunkSelected);

            // All values must return "s" which will be returned to the caller in a string array
            string s = null;

            // l is a general integer value for whatever
            int l;

            for (int i = 0; i < rd.chunkCount; i++)
            {
                BR.BaseStream.Position = baseOffset + (rd.reflexive.chunkSize * i) + bo.offset;
                IFPIO.ObjectEnum ObjType = bo.ObjectType;
                s = string.Empty;

                // For Block Indexes (Indices)
                if (bo is IFPIO.IFPInt && ((IFPIO.IFPInt)bo).entIndex != null)
                {
                    int value = -1;
                    switch (ObjType)
                    {
                        case IFPIO.ObjectEnum.Byte:
                            value = BR.ReadByte();
                            break;
                        case IFPIO.ObjectEnum.Short:
                            value = BR.ReadInt16();
                            break;
                        case IFPIO.ObjectEnum.UInt:
                            value = (int)BR.ReadUInt32();
                            break;
                        case IFPIO.ObjectEnum.UShort:
                            value = BR.ReadUInt16();
                            break;
                        case IFPIO.ObjectEnum.Int:
                            value = BR.ReadInt32();
                            break;
                    }

                    s += value + ": ";
                    if (((IFPIO.IFPInt)bo).entIndex.reflexiveLayer == "root")
                    {
                        BR.BaseStream.Position = ((IFPIO.IFPInt)bo).entIndex.ReflexiveOffset;
                        int count = BR.ReadInt32();
                        int ofs = BR.ReadInt32() - meta.magic - meta.offset;
                        ofs += (((IFPIO.IFPInt)bo).entIndex.ReflexiveSize * value) +
                               ((IFPIO.IFPInt)bo).entIndex.ItemOffset;
                        BR.BaseStream.Position = ofs;
                    }
                    else
                    {
                    }

                    switch (((IFPIO.IFPInt)bo).entIndex.ItemType.ToLower())
                    {
                        case "ident":
                            ObjType = IFPIO.ObjectEnum.Ident;
                            break;
                        case "string32":
                            ObjType = IFPIO.ObjectEnum.String32;
                            break;
                        case "string256":
                            ObjType = IFPIO.ObjectEnum.String256;
                            break;
                        case "stringid":
                            ObjType = IFPIO.ObjectEnum.StringID;
                            break;
                        default:
                            MessageBox.Show(
                                "Didn't add support for Index Block type \"" + ((IFPIO.IFPInt)bo).entIndex.ItemType +
                                "\". Please inform developer.");
                            ObjType = IFPIO.ObjectEnum.Unused;
                            break;
                    }
                }

                switch (ObjType)
                {
                    case IFPIO.ObjectEnum.Char_Enum:
                    case IFPIO.ObjectEnum.Enum:
                    case IFPIO.ObjectEnum.Long_Enum:
                        switch (((IFPIO.IFPEnum)bo).enumSize)
                        {
                            case 8:
                                l = BR.ReadByte();
                                s += ((IFPIO.BaseObject)((IFPIO.IFPEnum)bo).options[l]).name;
                                break;
                            case 16:
                                l = BR.ReadInt16();
                                s += ((IFPIO.BaseObject)((IFPIO.IFPEnum)bo).options[l]).name;
                                break;
                            case 32:
                                l = BR.ReadInt32();
                                s += ((IFPIO.BaseObject)((IFPIO.IFPEnum)bo).options[l]).name;
                                break;
                        }

                        break;

                        #region Bitmask Labels

                    case IFPIO.ObjectEnum.Byte_Flags:
                    case IFPIO.ObjectEnum.Word_Flags:
                    case IFPIO.ObjectEnum.Long_Flags:
                        l = 0;
                        switch (((IFPIO.Bitmask)bo).bitmaskSize)
                        {
                            case 8:
                                l = BR.ReadByte();
                                break;
                            case 16:
                                l = BR.ReadUInt16();
                                break;
                            case 32:
                                l = (int)BR.ReadUInt32();
                                break;
                        }

                        for (int ll = 0; ll < ((IFPIO.Bitmask)bo).bitmaskSize; ll++)
                        {
                            s += ((l >> ll) & 1) + s;
                        }

                        break;

                        #endregion

                        #region Ident Labels

                    case IFPIO.ObjectEnum.Ident:
                        l = BR.ReadInt32();
                        l = map.Functions.ForMeta.FindMetaByID(l);
                        if (l != -1)
                        {
                            s += "[" + map.MetaInfo.TagType[l] + "] " + map.FileNames.Name[l];
                        }
                        else
                        {
                            s += "[null] null";
                        }

                        break;

                        #endregion

                        #region Number Data Value Labels (Int, Float, Byt, UShort, etc)

                    case IFPIO.ObjectEnum.Byte:
                        l = BR.ReadByte();
                        s += (char)l + " (" + l + ")";
                        break;
                    case IFPIO.ObjectEnum.Float:
                    case IFPIO.ObjectEnum.Unknown:
                        s += BR.ReadSingle().ToString();
                        break;
                    case IFPIO.ObjectEnum.Short:
                        s += BR.ReadInt16().ToString();
                        break;
                    case IFPIO.ObjectEnum.UInt:
                        s += BR.ReadUInt32().ToString();
                        break;
                    case IFPIO.ObjectEnum.UShort:
                        s += BR.ReadUInt16().ToString();
                        break;
                    case IFPIO.ObjectEnum.Int:
                        s += BR.ReadInt32().ToString();
                        break;

                        #endregion

                        #region String ID Labels

                    case IFPIO.ObjectEnum.StringID:
                        short sidIndexer = BR.ReadInt16();
                        byte tempnull = BR.ReadByte();
                        byte sidLength = BR.ReadByte();
                        s += map.Strings.Name[sidIndexer];
                        if (s.Length != sidLength)
                        {
                            s = null;
                        }

                        break;

                        #endregion

                        #region String Labels

                    case IFPIO.ObjectEnum.UnicodeString256:
                    case IFPIO.ObjectEnum.UnicodeString64:
                        Encoding decode = Encoding.Unicode;
                        byte[] tempbytes = BR.ReadBytes(((IFPIO.IFPString)bo).size);
                        s += decode.GetString(tempbytes);
                        s = s.TrimEnd('\0');
                        break;
                    case IFPIO.ObjectEnum.String32:
                    case IFPIO.ObjectEnum.String256:
                        s += new string(BR.ReadChars(((IFPIO.IFPString)bo).size));
                        s = s.TrimEnd('\0');
                        break;

                        #endregion

                    default:
                        throw new Exception(
                            "Whoops! Forgot to code in \"" + bo.ObjectType + "\" data type for labels!\n" +
                            "Slap the programmer and tell him about this message!");
                        //return new string[rd.chunkCount];
                }

                ls.Add(s);
            }

            return ls.ToArray();
        }

        /// <summary>
        /// The refresh tree listing.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <remarks></remarks>
        private void refreshTreeListing(TreeNode parent)
        {
            if (((reflexiveData)parent.Tag).chunkCount == 0)
            {
                return;
            }

            foreach (TreeNode tn in parent.Nodes)
            {
                reflexiveData rd = (reflexiveData)tn.Tag;
                if (rd.inTagNumber != meta.TagIndex)
                {
                    map.OpenMap(MapTypes.Internal);
                    BR = map.BR;
                    BR.BaseStream.Position = ((reflexiveData)parent.Tag).baseOffset + rd.reflexive.offset;
                    if (((reflexiveData)parent.Tag).inTagNumber == meta.TagIndex)
                    {
                        BR.BaseStream.Position += meta.offset;
                    }
                }
                else
                {
                    BR = new BinaryReader(meta.MS);
                    BR.BaseStream.Position = ((reflexiveData)parent.Tag).baseOffset + rd.reflexive.offset;
                }

                rd.chunkCount = BR.ReadInt32();
                rd.baseOffset = BR.ReadInt32() - meta.magic + rd.chunkSelected * rd.reflexive.chunkSize;

                rd.inTagNumber = map.Functions.ForMeta.FindMetaByOffset(rd.baseOffset);
                if (rd.inTagNumber == meta.TagIndex)
                {
                    rd.baseOffset -= meta.offset;
                }
                else
                {
                    map.CloseMap();
                    if (rd.inTagNumber != -1)
                    {
                        tn.ForeColor = Color.Red;
                        tn.ToolTipText = "Data Source Located in:\n[" + map.MetaInfo.TagType[rd.inTagNumber].ToLower() +
                                         "] " + map.FileNames.Name[rd.inTagNumber].ToLower();
                    }
                }

                tn.Text = tn.Name + " [" + rd.chunkCount + "]";
                refreshTreeListing(tn);
            }
        }

        /// <summary>
        /// The timer 1_ tick.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            panelInfoPane.Visible = false;
        }

        /// <summary>
        /// The tree view tag reflexives_ after select.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void treeViewTagReflexives_AfterSelect(object sender, TreeViewEventArgs e)
        {
            TreeNode tn = e.Node;

            loadControls(tn);

            toolStripNavigation.Items.Clear();
            while (tn.Parent != null)
            {
                addReflexive(toolStripNavigation, tn.Text);
                toolStripNavigation.Items[1].Name = tn.Name;

                reflexiveData rd = (reflexiveData)tn.Tag;

                // will throw exception if in the [MAIN] reflexive, so just ignore
                /*
                try
                {
                    if (ctl.name == rd.reflexive.label && !labelFound)
                    {
                        labelFound = true;
                        panelMetaEditor.Controls[0].Controls[1].LostFocus += new EventHandler(WinMEControl_LostFocus);
                    }
                }
                catch { }
                */
                string[] s = loadLabels(rd);

                // Create reflexive count comboBox listings
                for (int i = 0; i < rd.chunkCount; i++)
                {
                    if (s[i] != null)
                    {
                        ((ToolStripComboBox)toolStripNavigation.Items[2]).Items.Add(i + " : \"" + s[i] + "\"");
                    }
                    else
                    {
                        ((ToolStripComboBox)toolStripNavigation.Items[2]).Items.Add(i);
                    }
                }

                if (rd.chunkCount != 0)
                {
                    if (rd.chunkSelected > rd.chunkCount)
                    {
                        rd.chunkSelected = rd.chunkCount - 1;
                    }

                    try
                    {
                        ((ToolStripComboBox)toolStripNavigation.Items[2]).SelectedIndex = rd.chunkSelected;
                    }
                    catch
                    {
                    }
                }

                tn = tn.Parent;
            }

            panelMetaEditor.Enabled = ((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).chunkCount != 0;

            // If the data is not in this tag, just disbale editing of it. Make it done in parent tag ONLY!
            if (meta.TagIndex != ((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).inTagNumber)
            {
                panelMetaEditor.Enabled = false;
            }

            btnSave.Enabled = panelMetaEditor.Enabled;
            btnReset.Enabled = panelMetaEditor.Enabled;
            btnPeek.Enabled = panelMetaEditor.Enabled;
            btnPoke.Enabled = panelMetaEditor.Enabled;

            /*******/
            btnReset.Enabled = false;
            btnPeek.Enabled = false;
            btnPoke.Enabled = false;

            /*******/
            if (((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).inTagNumber != -1 &&
                ((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).inTagNumber != meta.TagIndex)
            {
                showInfoBox(
                    "This reflexive resides within \"[" +
                    map.MetaInfo.TagType[((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).inTagNumber] + "] " +
                    map.FileNames.Name[((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).inTagNumber] +
                    "\"\r\nYou must edit these values in the parent tag." +
                    "\r\nDouble click the reflexive in the tree on the left to load the reflexive in the given tag.", 
                    0);
            }
            else
            {
                showInfoBox(string.Empty, 1);
            }
        }

        /// <summary>
        /// The tree view tag reflexives_ double click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void treeViewTagReflexives_DoubleClick(object sender, EventArgs e)
        {
            if (((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).inTagNumber == meta.TagIndex)
            {
                return;
            }

            WinMetaEditor wME = new WinMetaEditor(
                (MapForm)this.Owner, ((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).inTagNumber, map);
            wME.gotoOffset(((reflexiveData)treeViewTagReflexives.SelectedNode.Tag).baseOffset);
            wME.Show();
            wME.Focus();
        }

        /// <summary>
        /// The tree view tag reflexives_ draw node.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void treeViewTagReflexives_DrawNode(object sender, DrawTreeNodeEventArgs e)
        {
            if ((e.Node.ForeColor == Color.Red) & ((e.State & TreeNodeStates.Selected) != 0))
            {
                e.Graphics.DrawString(
                    e.Node.Text, 
                    e.Node.TreeView.Font, 
                    Brushes.PaleVioletRed, 
                    new Rectangle(e.Bounds.Left, e.Bounds.Top, e.Bounds.Width + 10, e.Bounds.Height - 2));
            }
                
                // Paint text of other nodes in default color
            else
            {
                e.DrawDefault = true;
            }
        }

        /// <summary>
        /// The tsbc_ drop down.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void tsbc_DropDown(object sender, EventArgs e)
        {
            ToolStripComboBox tsbc = (ToolStripComboBox)sender;
            int desiredWidth = 49;
            for (int i = 0; i < tsbc.Items.Count; i++)
            {
                if (TextRenderer.MeasureText(tsbc.Items[i].ToString(), tsbc.Font).Width + 8 > desiredWidth)
                {
                    desiredWidth = TextRenderer.MeasureText(tsbc.Items[i].ToString(), tsbc.Font).Width + 16;
                }
            }

            tsbc.Size = new Size(desiredWidth, 21);
        }

        /// <summary>
        /// The tsbc_ drop down closed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void tsbc_DropDownClosed(object sender, EventArgs e)
        {
            ToolStripComboBox tsbc = (ToolStripComboBox)sender;
            tsbc.Size = new Size(49, 21);
        }

        /// <summary>
        /// The tsbc_ selected index changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void tsbc_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!((ToolStripItem)sender).Enabled)
            {
                return;
            }

            

            ToolStripComboBox tscb = (ToolStripComboBox)sender;
            int tsGroupNumber = (toolStripNavigation.Items.IndexOf(tscb) - 2) / 3;
            TreeNode tn = treeViewTagReflexives.SelectedNode;
            List<TreeNode> tns = new List<TreeNode>();
            tns.Add(tn);
            for (int ii = tsGroupNumber; ii < ((toolStripNavigation.Items.Count - 2) / 3); ii++)
            {
                tn = tn.Parent;
                tns.Insert(0, tn);
            }

            reflexiveData rd = (reflexiveData)tn.Tag;
            rd.chunkSelected = tscb.SelectedIndex;

            

            // tn = TreeNode that was changed
            // tns = all nodes from selected node to changed node
            // tsGroupNumber = ComboBox # that was changed (from Left->Right)

            // Update sub-Combo Box listings for lead Combo Boxes that were changed
            refreshTreeListing(treeViewTagReflexives.Nodes[0]);

            // Update tree listings counters
            int levelsUp = treeViewTagReflexives.SelectedNode.Level - rd.node.Level;
            for (int j = tsGroupNumber; j <= tsGroupNumber + levelsUp; j++)
            {
                rd = (reflexiveData)tns[j - tsGroupNumber].Tag;
                toolStripNavigation.Items[j * 3 + 1].Text = toolStripNavigation.Items[j * 3 + 1].Name + " [" +
                                                            rd.chunkCount + "]";
                int selection = ((ToolStripComboBox)toolStripNavigation.Items[j * 3 + 2]).SelectedIndex;
                int count = ((ToolStripComboBox)toolStripNavigation.Items[j * 3 + 2]).Items.Count;

                #region check Combo Box and make sure it contains the proper amount of numbers

                if (rd.chunkCount > count)
                {
                    for (int ii = count; ii < rd.chunkCount; ii++)
                    {
                        ((ToolStripComboBox)toolStripNavigation.Items[j * 3 + 2]).Items.Add(ii);
                    }
                }
                else if (rd.chunkCount < count && rd.chunkCount > 0)
                {
                    for (int ii = count - 1; ii > rd.chunkCount - 1; ii--)
                    {
                        ((ToolStripComboBox)toolStripNavigation.Items[j * 3 + 2]).Items.RemoveAt(ii);
                    }

                    if (((ToolStripComboBox)toolStripNavigation.Items[j * 3 + 2]).SelectedIndex == -1)
                    {
                        toolStripNavigation.Items[j * 3 + 2].Enabled = false;
                        ((ToolStripComboBox)toolStripNavigation.Items[j * 3 + 2]).SelectedIndex = rd.chunkCount - 1;
                        toolStripNavigation.Items[j * 3 + 2].Enabled = true;
                    }
                }
                else if (rd.chunkCount == 0)
                {
                    ((ToolStripComboBox)toolStripNavigation.Items[j * 3 + 2]).Items.Clear();
                }

                #endregion
            }

            tn = treeViewTagReflexives.SelectedNode;

            if (((reflexiveData)tn.Tag).chunkCount > 0)
            {
                ReloadMetaForSameReflexive(((reflexiveData)tn.Tag).baseOffset);
            }
        }

        #endregion

        /// <summary>
        /// The reflexive data.
        /// </summary>
        /// <remarks></remarks>
        private class reflexiveData
        {
            #region Constants and Fields

            /// <summary>
            /// The base offset.
            /// </summary>
            public int baseOffset;

            /// <summary>
            /// The chunk count.
            /// </summary>
            public int chunkCount;

            /// <summary>
            /// The chunk selected.
            /// </summary>
            public int chunkSelected;

            /// <summary>
            /// The in tag number.
            /// </summary>
            public int inTagNumber;

            /// <summary>
            /// The node.
            /// </summary>
            public TreeNode node;

            /// <summary>
            /// The reflexive.
            /// </summary>
            public IFPIO.Reflexive reflexive;

            /// <summary>
            /// The ts items.
            /// </summary>
            //public ToolStripItem[] tsItems;

            #endregion

            #region Constructors and Destructors

            /// <summary>
            /// Initializes a new instance of the <see cref="reflexiveData"/> class.
            /// </summary>
            /// <remarks></remarks>
            public reflexiveData()
            {
                node = null;
                reflexive = null;
                baseOffset = -1;
                chunkCount = 0;
                chunkSelected = -1;
                inTagNumber = -1;
            }

            #endregion
        }
    }
}