// --------------------------------------------------------------------------------------------------------------------
// <copyright file="SID.cs" company="">
//   
// </copyright>
// <summary>
//   The sid.
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace entity.MetaEditor2
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Windows.Forms;

    using entity.MapForms;
    using entity.MetaFuncs;

    using Globals;

    using HaloMap.Map;
    using HaloMap.RealTimeHalo;

    /// <summary>
    /// The sid.
    /// </summary>
    /// <remarks></remarks>
    public partial class SID : BaseField
    {
        // public int LineNumber;
        // private int mapIndex;
        // public int chunkOffset;
        // public int offsetInMap;
        // public string EntName = "Error in getting plugin element name";
        #region Constants and Fields

        /// <summary>
        /// The sid indexer.
        /// </summary>
        public int sidIndexer;

        /// <summary>
        /// The w m_ paint.
        /// </summary>
        private const short WM_PAINT = 0x00f;

        /// <summary>
        /// The sid indexer list.
        /// </summary>
        private readonly List<int> sidIndexerList = new List<int>(0);

        /// <summary>
        /// The add events.
        /// </summary>
        private bool AddEvents = true;

        /// <summary>
        /// The is nulled out reflexive.
        /// </summary>
        private bool isNulledOutReflexive = true;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SID"/> class.
        /// </summary>
        /// <param name="iEntName">Name of the i ent.</param>
        /// <param name="map">The map.</param>
        /// <param name="iOffsetInChunk">The i offset in chunk.</param>
        /// <param name="iLineNumber">The i line number.</param>
        /// <remarks></remarks>
        public SID(string iEntName, Map map, int iOffsetInChunk, int iLineNumber)
        {
            this.LineNumber = iLineNumber;
            this.chunkOffset = iOffsetInChunk;
            mapIndex = map;
            this.EntName = iEntName;
            InitializeComponent();
            this.Size = this.PreferredSize;
            this.Dock = DockStyle.Top;
            this.Controls[0].Text = EntName;
            this.AutoSize = false;
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// The base field_ leave.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        public override void BaseField_Leave(object sender, EventArgs e)
        {
            // Check for typed value
            SID sid = (SID)sender;
            if (sid.comboBox1.Text != mapIndex.Strings.Name[sid.sidIndexer])
            {
                for (int i = 0; i < mapIndex.Strings.Name.Length; i++)
                {
                    if (mapIndex.Strings.Name[i].ToLower() == sid.comboBox1.Text.ToLower())
                    {
                        sid.sidIndexer = i;
                        break;
                    }
                }

                sid.comboBox1.Text = mapIndex.Strings.Name[sid.sidIndexer];
            }

            // if (this.AutoSave)
            // this.Save();
        }

        /// <summary>
        /// The poke.
        /// </summary>
        /// <remarks></remarks>
        public void Poke()
        {
            uint Address = (uint)(this.offsetInMap + mapIndex.SelectedMeta.magic);
            try
            {
                uint StringID =
                    (uint)(((ushort)this.sidIndexer) | ((byte)mapIndex.Strings.Length[this.sidIndexer] << 24));
                RTH_Imports.Poke(Address, StringID, 32);
            }
            catch (Exception e)
            {
                Global.ShowErrorMsg(
                    "Net: Something is wrong with this Sid " + this.EntName + " Offset " + this.chunkOffset, e);
            }
        }

        /// <summary>
        /// The populate.
        /// </summary>
        /// <param name="iOffset">The i offset.</param>
        /// <param name="useMemoryStream">The use memory stream.</param>
        /// <remarks></remarks>
        public void Populate(int iOffset, bool useMemoryStream)
        {
            this.isNulledOutReflexive = false;
            BinaryReader BR = new BinaryReader(mapIndex.SelectedMeta.MS);

            // set offsets
            BR.BaseStream.Position = iOffset + this.chunkOffset;
            this.offsetInMap = iOffset + this.chunkOffset;

            // If we need to read / save tag info directly to file...
            if (!useMemoryStream)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                BR = mapIndex.BR;
                BR.BaseStream.Position = this.offsetInMap;
            }
            else
            {
                this.offsetInMap += mapIndex.SelectedMeta.offset;
            }

            this.sidIndexer = BR.ReadInt16();
            byte tempnull = BR.ReadByte();
            byte sidLength = BR.ReadByte();

            // ...and then close the file once we are done!
            if (!useMemoryStream)
            {
                mapIndex.CloseMap();
            }

            try
            {
                string s = mapIndex.Strings.Name[this.sidIndexer];
                if (s.Length == sidLength)
                {
                    this.Controls[1].Text = s;
                }
                else
                {
                    this.Controls[1].Text = string.Empty;
                }
            }
            catch
            {
                this.Controls[1].Text = "error reading sid";
            }

            if (AddEvents)
            {
                this.Controls[1].TextChanged += this.comboBox1_TextChanged;
                AddEvents = false;
            }
        }

        /// <summary>
        /// The save.
        /// </summary>
        /// <exception cref="NotImplementedException">
        ///   </exception>
        /// <remarks></remarks>
        public override void Save()
        {
            if (this.isNulledOutReflexive)
            {
                return;
            }

            bool openedMap = false;
            if (mapIndex.isOpen == false)
            {
                mapIndex.OpenMap(MapTypes.Internal);
                openedMap = true;
            }

            try
            {
                mapIndex.BW.BaseStream.Position = this.offsetInMap;
                mapIndex.BW.Write((short)this.sidIndexer);
                mapIndex.BW.BaseStream.Position += 1;
                mapIndex.BW.Write((byte)mapIndex.Strings.Length[this.sidIndexer]);
            }
            catch (Exception e)
            {
                Global.ShowErrorMsg(
                    "Something is wrong with this Sid " + this.EntName + " Offset " + this.chunkOffset, e);
            }

            if (openedMap)
            {
                mapIndex.CloseMap();
            }
        }

        /// <summary>
        /// The set focus.
        /// </summary>
        /// <param name="LineToCheck">The line to check.</param>
        /// <remarks></remarks>
        public void SetFocus(int LineToCheck)
        {
            if (this.LineNumber == LineToCheck)
            {
                this.Focus();
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// The sid loader_ drop down.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void SIDLoader_DropDown(object sender, EventArgs e)
        {
            string tempSidString = ((ComboBox)sender).Text;
            SID sid = (SID)((Control)sender).Parent;

            MapForm mf = (MapForm)this.ParentForm.Owner;

            if (mf != null)
            {
                if (mf.sSwap == null)
                {
                    mf.sSwap = new MEStringsSelector(mapIndex.Strings.Name);
                }

                mf.sSwap.SelectedID = sid.sidIndexer;
                mf.sSwap.ShowDialog();

                // this.Enabled = true;
                if (sid.sidIndexer != mf.sSwap.SelectedID)
                {
                    sid.sidIndexer = mf.sSwap.SelectedID;
                    ((ComboBox)sender).SelectedIndex = -1;
                    ((ComboBox)sender).Text = mapIndex.Strings.Name[mf.sSwap.SelectedID];
                }
            }
            else
            {
                ((ComboBox)sender).Items.Clear();
                this.sidIndexerList.Clear();
                ((ComboBox)sender).Items.Add(string.Empty);
                this.sidIndexerList.Add(0);
                for (int counter = 0; counter < mapIndex.Strings.Name.Length; counter++)
                {
                    if (mapIndex.Strings.Name[counter].Contains(tempSidString))
                    {
                        ((ComboBox)sender).Items.Add(mapIndex.Strings.Name[counter]);
                        string xe = mapIndex.Strings.Name[counter];
                        this.sidIndexerList.Add(counter);
                        if (counter == sidIndexer)
                        {
                            ((ComboBox)sender).SelectedIndex = this.sidIndexerList.Count - 1;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// The sid loader_ drop down close.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void SIDLoader_DropDownClose(object sender, EventArgs e)
        {
            if (((ComboBox)sender).SelectedIndex != -1)
            {
                try
                {
                    sidIndexer = sidIndexerList[((ComboBox)sender).SelectedIndex];
                }
                catch
                {
                    sidIndexer = 0;
                }
            }
        }

        /// <summary>
        /// The combo box 1_ text changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The e.</param>
        /// <remarks></remarks>
        private void comboBox1_TextChanged(object sender, EventArgs e)
        {
        }

        #endregion
    }
}